/*
**  misc.c
**
**  Miscellaneous routines that don't really belong in any
**  of the other modules.
**
**  12/14/90 bf@hp
*/

#include "misc.h"
#include "sect.h"
#include "ship.h"
#include "plane.h"

/* ________________________________________________________________
**|                                                                |
**| int chrinstr(str,c) -					   |
**|								   |
**| Return TRUE if char `c' is in string `str'                     |
**|________________________________________________________________|
*/
int chrinstr(str,c)
char *str,c;
{
    while (*str && (*str != c)) str++;
    return ((int)(*str));
}

/* ________________________________________________________________
**|                                                                |
**| char *aan(s) -					           |
**|								   |
**| Return ptr to `a' or `an' article string, whichever is         |
**| appropriate to put before string `s'                           |
**|________________________________________________________________|
*/
char *aan(s)
char *s;
{
    static char buf[4];
    strcpy(buf,"a");
    if (chrinstr("AEIOUaeiou",*s)) 
	strcat(buf,"n");
    return(buf);
}

/* ________________________________________________________________
**|                                                                |
**| stypeinlist(list,stype) -                                      |
**|								   |
**| Return TRUE if sector type `stype' is in a sector              |
**| designator string `list'                                       |
**|________________________________________________________________|
*/
int stypeinlist(list,stype) 
char *list;
int stype;
{
    int i,neg;
    if (neg=(*list=='~')) list++;
    i = chrinstr(list,dchr[stype].d_mnem);
    return (neg ? !i : i);
}

/* ________________________________________________________________
**|                                                                |
**| fmslist(slist) -                                               |
**|								   |
**| Print names of sectors in a sector designator list.            |
**| Note that this uses a static char buffer that's only 256 bytes |
**| long.  This means you must use the return string right away,   |
**| and that the list of sector designators musn't expand into     |
**| a list that approaches 256 bytes in length (there is no error  |
**| checking).                                                     |
**|________________________________________________________________|
*/
char *fmslist(slist) 
char *slist;
{
    static char buf[512];
    int i,n,first,neg;
    *buf = 0;
    if (neg=(*slist=='~')) slist++;
    /* count up how many sector types we'll print */
    for (i=n=0;i<=SCT_MAXDEF;i++)
	if (chrinstr(slist,dchr[i].d_mnem)) n++;
    /* now pass thru them again and print them */
    if ((first=n)) {
	if (neg) strcat(buf,"a sector other than ");
	for (i=0;n;i++) {
	    if (chrinstr(slist,dchr[i].d_mnem)) {
		if (n == first) {
		    strcat(buf,aan(dchr[i].d_name));
		    strcat(buf," ");
		}
		strcat(buf,dchr[i].d_name);
		if (--n) strcat(buf,", ");
		if (n == 1) strcat(buf,"or ");
	    }
	}
    }
    return(buf);
}


/* _____________________________________________________________________
**|                                                                     |
**| planefitsship - return TRUE if a plane with the given plane flags   |
**|                 can be landed on a ship with the given ship flags   |
**|_____________________________________________________________________|
*/

int planefitsship(pflags,sflags)
int pflags,sflags;
{
    if ((pflags & P_H) && (sflags & (M_FLY|M_VTOL|M_HELI))) return(1);
    if ((pflags & P_V) && (sflags & (M_FLY|M_VTOL))) return(1);
    if ((pflags & P_L) && (sflags & (M_FLY))) return(1);
    if ((pflags & P_M) && (sflags & (M_MSL)) && (pflags & P_L)) return(1);
    return(0);
}

