static char *RCSid = "$Header: /sequent2/empire/EMP/lib/global/RCS/plane.c,v 1.1 89/09/02 18:30:21 mr-frog Exp $";

/*
**  plane.c
**
** For planes:
**  P_B	 Bombing factor only for bombing 
**  P_T	 Tactical bombing ability 
**  P_F	 Interceptor/escort 
**  P_C	 Bombing factor for transport only 
**  P_V	 Vtol ability (planes and helicopters) 
**  P_M	 Missile, used only once, can't be intercepted 
**  P_L	 Light (can be carried on a ship or vehicle)
**  P_X	 Stealth ability 
**  P_H  Helicopter 
**  P_U  Can locate subs 
**  P_HA High altitude aircraft (immune to flak)
**
** Note:
**  Tactical missiles (P_M|P_T) can be launched at specific targets;
**  Nontactical missiles can be launched only at a specific sector.
**  Helicopters must have Vtol (P_V) capability.
**
** For satellites:
**  P_E  Enhanced capabilities
**  P_S	 Spy ability 
**  P_I	 Imaging ability (advanced spying) 
**  P_O	 Orbital ability (a satellite) 
*/

#include "misc.h"
#include "plane.h"

struct plchrstr plchr[] = {
/*       name          lcm hcm $$$ tech acc load att/def ran mil gas flags */
#ifdef STANDARDPLANES
	"fighter 1",	 8, 2, 400, 40, 30,  1,  3,  2,  6,  1,  1, P_V|P_T|P_F|P_L,
	"lt bomber 1",	10, 3, 550, 40, 30,  2,  1,  2,  8,  2,  1, P_V|P_T|P_B|P_L,
	"transport 1",	12, 5, 800, 45,  0,  7,  0,  1, 15,  2,  2, P_C,
	"hvy bomber 1",	20, 6,1100, 50, 85,  6,  2,  3, 15,  3,  2, P_B,
	"long bomber 1",20, 6,1100, 60,120,  2,  0,  1, 30,  4,  4, P_B,
	"fighter 2",	10, 6, 800, 70, 25,  1,  6,  4,  8,  1,  1, P_T|P_F|P_L,
	"lt bomber 2",	12, 8, 900, 70, 25,  3,  2,  3, 12,  2,  1, P_T|P_B|P_L,
	"escort 2",	12, 5, 850, 75,  0,  0,  4,  4, 16,  1,  2, P_F|P_L,
	"chopper 1",	10, 4, 900, 75,  0,  5,  0,  2,  8,  1,  1, P_C|P_V|P_H|P_L,
	"transport 2",	14,10,1200, 80,  0,  9,  0,  2, 20,  3,  3, P_C,
	"long transport",14,10,1200,80,  0,  3,  0,  1, 35,  4,  5, P_C,
	"hvy bomber 2",	20,15,2000, 90, 75, 10,  3,  5, 20,  9,  3, P_B,
	"long bomber 2",20,15,2000, 95, 75,  4,  2,  2, 35,  6,  5, P_B,
	"jet fighter",	16, 5,2200,120, 40,  2, 12,  8,  9,  1,  4, P_T|P_F|P_L,
	"jet lt bomber",14, 7,1500,120, 20,  5,  5,  8, 15,  2,  5, P_T|P_B|P_L,
	"jet hvy bomber",35,20,5200,140,65, 16,  4, 10, 40,  8,  8, P_B,
	"jet transport",30,20,2100,140,  0, 16,  0,  5, 40,  4,  7, P_C,
	"jet fighter 2",16, 5,2000,200, 30,  2, 12,  8,  9,  1,  5, P_T|P_F|P_V|P_L,
	"jet f/b",	12, 5,3500,210, 40,  8, 10, 10, 32,  2,  6, P_B|P_T,
	"spyplane 1",	15,10,3000,190,  0,  0,  0,  7, 40,  1,  5, P_S,
	"spyplane 2",	18,13,4500,275,  0,  0,  0, 12, 60,  2,  8, P_S|P_I,
	"stealth bomber",45,35,9500,350,60, 12,  3, 12, 36,  3,  4, P_B|P_X,
	"stealth fighter",35,25,9500,350,30, 3, 15, 12, 20,  1,  3, P_T|P_F|P_X,
	"chopper 2",	15, 8,1000,180, 10,  3,  5,  5, 12,  1,  2, P_T|P_B|P_V|P_H|P_L,
	"chopper 3",	15,12,1100,180,  0,  6,  2,  3, 20,  4,  3, P_C|P_V|P_H|P_L,
	"chopper 4",	15,12,1500,265, 10,  5,  7,  7, 18,  2,  3, P_T|P_B|P_V|P_H|P_L,
	"ssm 1",	10, 6, 500,160, 60,  5,  0,  5,  6,  0,  0, P_M|P_L,
	"ssm 2",	12, 8, 600,200, 50,  7,  0,  6,  8,  0,  0, P_M|P_L,
/* XXX fix these later */
	"irbm 1",	15,15,1000,250, 60,  5,  0,  3, 15,  0,  7, P_M,
	"irbm 2",	18,18,2000,270, 50,  7,  0,  3, 18,  0,  8, P_M,
	"icbm 1",	30,30,3000,280, 70,  7,  0,  3, 40,  0,  9, P_M,
	"icbm 2",	45,45,4000,320, 50, 10,  0,  6, 50,  0, 10, P_M,
	"icbm 3",	60,60,5000,400, 20, 15,  0, 10, 60,  0, 11, P_M,
	"icbm 4",     100,100,9000,450, 20, 30,  0, 50 ,60,  0, 12, P_M,
	"slbm 1",	25,25,3000,260, 70,  7,  0,  3, 19,  0,  6, P_M|P_L,
	"slbm 2",	25,25,5000,360, 70, 11,  0,  3, 40,  0,  7, P_M|P_L,
	"sam 1",	25,25,300, 130, 30,  4, 12,  1,  5,  0,  6, P_M|P_F,
	"sam 2",	35,35,500, 230, 20,  4, 12,  1,  5,  0,  6, P_M|P_F|P_L,
	"sam 3",	45,45,1600,380, 10,  4, 14,  1, 15,  0,  6, P_M|P_F,
	"sam 4",	45,45,2000,480, 10,  4, 14,  1, 15,  0,  6, P_M|P_F|P_L,
	"landsat 1"	,20,20,8000,205, 0,  0,  0,  2, 99,  0,  9, P_O,
	"landsat 2"	,25,25,9500,305, 0,  0,  0,  2, 99,  0,  9, P_O|P_I,
	"spysat 1"	,30,30,25000,405,0,  0,  0,  2, 99,  0,  9, P_O|P_S,
	"spysat 2"	,35,35,30000,505,0,  0,  0,  2, 99,  0,  9, P_O|P_S|P_I,
	"anti-sat 1",  13, 8,  700,185, 70,  6,  0,  6,  8,  0,  4, P_O|P_M|P_L,
	"anti-sat 2",  15, 8,  800,235, 60,  8,  0,  6, 12,  0,  5, P_O|P_M|P_L,
	"anti-sat 3",  18, 8,  900,285, 50,  8,  0,  6, 20,  0,  6, P_O|P_M|P_L,
	"anti-sat 4",  35,35, 2500,335, 40, 16,  0,  6, 99,  0,  9, P_O|P_M,
	"asw chopper 1",15,12,1200,170,  0,  3,  2,  3, 18,  2,  3, P_V|P_H|P_L|P_U,
	"asw chopper 2",15,12,1500,220, 10,  3,  7,  7, 16,  2,  3, P_T|P_B|P_V|P_H|P_L|P_U,
#else

/* WALTER'S PLANES */

/* tech 0-99 */
/*   name         lcm  hcm   $$$ tech acc load att def rng mil gas  flags */
"fighter 1",        8,   2,  400,  40, 30,   1,  3,  2,  6,  1,  1, P_T |P_F |P_V |P_L,
"lt bomber 1",     10,   3,  550,  40, 30,   2,  1,  2,  8,  2,  1, P_B |P_T |P_V |P_L,
"transport 1",     12,   5,  800,  45,  0,   7,  0,  1, 15,  2,  2, P_C,
"hvy bomber 1",    20,   6, 1100,  50, 85,   6,  2,  3, 15,  3,  2, P_B,
"long bomber 1",   20,   6, 1100,  70,120,   2,  0,  1, 30,  4,  4, P_B,
"long transport",  14,  10, 1200,  80,  0,   3,  0,  1, 35,  4,  5, P_C,
"transport 2",     14,  10, 1200,  80,  0,   9,  0,  2, 20,  3,  3, P_C,
"fighter 2",       10,   6,  800,  80, 25,   1,  6,  4,  8,  1,  1, P_T |P_F |P_L,
"escort 1",        12,   5,  850,  80,  0,   0,  4,  4, 16,  1,  2, P_F |P_L ,
"lt bomber 2",     12,   8,  900,  80, 25,   3,  2,  3, 12,  2,  1, P_B |P_T |P_L,
"chopper 1",       10,   4,  450,  90,  0,   5,  0,  2,  8,  1,  1, P_C |P_V |P_L |P_H,

/* tech 100-199 */
/*   name         lcm  hcm   $$$ tech acc load att def rng mil gas  flags */
"chopper 2",       15,   8,  900, 120, 10,   3,  5,  5, 12,  1,  2, P_B |P_T |P_V |P_L |P_H,
"hvy bomber 2",    20,  15, 2000, 120, 75,  10,  3,  5, 20,  9,  3, P_B,
"long bomber 2",   20,  15, 2000, 120, 75,   4,  2,  2, 35,  6,  5, P_B,
"ssm 1",           10,   6,  500, 150, 60,   5,  0,  5,  6,  0,  0, P_M |P_L,
"chopper 3",       15,  12,  600, 180,  0,   6,  2,  3, 20,  4,  3, P_C |P_V |P_L |P_H,

/* tech 200-299 */
/*   name         lcm  hcm   $$$ tech acc load att def rng mil gas  flags */
"jet lt bomber",   14,   7, 1500, 200, 20,   5,  5,  8, 15,  2,  5, P_B |P_T |P_L,
"jet transport",   30,  20, 2100, 200,  0,  16,  0,  5, 40,  4,  7, P_C,
"jet fighter 1",   16,   5, 2200, 200, 40,   2, 12,  8,  9,  1,  4, P_T |P_F |P_L,
"escort 2",        20,   5, 2400, 200,  0,   0,  8,  8, 40,  1,  4, P_F ,
"jet hvy bomber",  35,  20, 5200, 200, 65,  16,  4, 10, 40,  8,  8, P_B |P_HA,
"ssm 2",           12,   8,  600, 200, 50,   7,  0,  6,  8,  0,  0, P_M |P_L,
"sam 1",           25,  25,  300, 210, 30,   4, 12,  1,  5,  0,  0, P_F |P_M,
"asw chopper 1",   15,  12, 1200, 220,  0,   3,  2,  3, 18,  2,  3, P_V |P_L |P_H |P_U,
"spyplane 1",      15,  10, 3000, 225,  0,   0,  0,  7, 40,  1,  5, P_S |P_HA,
"anti-sat 1",      13,   8,  700, 225, 70,   6,  0,  6,  8,  0,  4, P_M |P_L |P_O,
"landsat 1",       20,  20, 8000, 225,  0,   0,  0,  2, 99,  0,  9, P_O,
"asw chopper 2",   15,  12, 1500, 250, 10,   3,  7,  7, 16,  2,  3, P_B |P_T |P_V |P_L |P_H |P_U |P_MS,
"sam 2",           35,  35,  500, 250, 20,   4, 12,  1,  5,  0,  0, P_F |P_M |P_L,
"anti-sat 2",      15,   8,  800, 250, 60,   8,  0,  6, 12,  0,  5, P_M |P_L |P_O,
"chopper 4",       15,  12, 1500, 265, 10,   5,  7,  7, 18,  2,  3, P_B |P_T |P_V |P_L |P_H |P_MS,
"anti-sat 3",      18,   8,  900, 285, 50,   8,  0,  6, 20,  0,  6, P_M |P_L |P_O,

/* tech 300-399 */
/*   name         lcm  hcm   $$$ tech acc load att def rng mil gas  flags */
"jet fighter 2",   16,   5, 3500, 300, 30,   2, 12,  8,  9,  1,  5, P_T |P_F |P_V |P_L,
"spyplane 2",      18,  13, 4500, 300,  0,   0,  0, 12, 60,  2,  8, P_S |P_I |P_HA,
"landsat 2",       25,  25, 9500, 305,  0,   0,  0,  2, 99,  0,  9, P_I |P_O,
"irbm 1",          15,  15, 1000, 325, 60,   5,  0,  3, 15,  0,  7, P_M,
"anti-sat 4",      35,  35, 2500, 335, 40,  16,  0,  6, 99,  0,  9, P_M |P_O,
"irbm 2",          18,  18, 2000, 350, 50,   7,  0,  3, 18,  0,  8, P_M,
"icbm 1",          30,  30, 3000, 350, 70,   7,  0,  3, 40,  0,  9, P_M,
"slbm 1",          25,  25, 3000, 350, 70,   7,  0,  3, 19,  0,  6, P_M |P_L,
"jet escort 1",    15,   8, 3000, 375,  0,   0, 14,  8, 45,  2,  6, P_F,
"jet escort 2",    15,   8, 3000, 375,  0,   0, 12,  8, 30,  2,  6, P_F |P_L,
"jet f/b",         12,   5, 3500, 375, 40,   8, 10, 10, 32,  2,  6, P_B |P_T,
"jet fighter 3",   20,   8, 5000, 375, 28,   4, 30, 10, 10,  2,  5, P_T |P_F |P_L,
"sam 3",           45,  45, 1600, 380, 10,   4, 14,  1, 15,  0,  0, P_F |P_M,

/* tech 400-499 */
/*   name         lcm  hcm   $$$ tech acc load att def rng mil gas  flags */
"icbm 2",          45,  45, 4000, 400, 50,  10,  0,  6, 50,  0, 10, P_M,
"slbm 2",          25,  25, 5000, 400, 70,  11,  0,  3, 40,  0,  7, P_M |P_L,
"spysat 1",        30,  30,25000, 405,  0,   0,  0,  2, 99,  0,  9, P_S |P_O,
"icbm 3",          60,  60, 5000, 450, 20,  15,  0, 10, 60,  0, 11, P_M,
"sam 4",           45,  45, 2000, 480, 10,   4, 14,  1, 15,  0,  0, P_F |P_M |P_L,

/* tech 500-599 */
/*   name         lcm  hcm   $$$ tech acc load att def rng mil gas  flags */
"icbm 4",         100, 100, 9000, 500, 20,  30,  0, 50, 60,  0, 12, P_M,
"stealth bomber",  45,  35, 9500, 500, 60,  12,  3, 12, 36,  3,  4, P_B |P_X,
"stealth fighter", 35,  25, 9500, 500, 30,   3, 12, 18, 20,  1,  3, P_T |P_F |P_X,
"spysat 2",        35,  35,30000, 505,  0,   0,  0,  2, 99,  0,  9, P_S |P_I |P_O,
"hyp transport 1", 65,  35,12500, 530,  0,  30,  0,  5, 90, 10, 20, P_C,
"hyp f/b 1",       70,  40,14000, 540, 35,  15, 10, 12, 90,  7, 20, P_B |P_T,
"icbm 5",         120, 120,15000, 580, 20,  50,  0, 50, 95,  0, 20, P_M,
"hyp fighter 1",   35,  18,14000, 540, 20,   4, 60, 20, 25,  2,  8, P_T |P_F,
"hyp fighter 2",   35,  18,15500, 550, 20,   3, 50, 20, 20,  2,  8, P_T |P_F |P_L |P_V,
"sam 5",           50,  45, 2800, 555,  8,   4, 16,  1, 20,  0,  0, P_F |P_M |P_L,
"hyp transport 2", 50,  25,15200, 570,  0,  36,  0, 18, 90, 10, 20, P_C,
"hyp f/b 2",       55,  30,16000, 580, 25,  18, 35, 20, 90,  3, 20, P_B |P_T,
"landsat 3",       30,  30,12000, 580,  0,   0,  0,  2, 99,  0,  9, P_E |P_I |P_O,
"spysat 3",        40,  35,32500, 590,  0,   0,  0,  2, 99,  0,  9, P_E |P_S |P_I |P_O,

/* cruise missiles */
/*   name         lcm  hcm   $$$ tech acc load att def rng mil gas  flags */
"cruise 1",        11,   6, 1000, 385, 30,   6,  0,  5,  7,  0,  0, P_T |P_M |P_L,
"cruise 2",        10,   5, 1250, 420, 22,   8,  0,  6,  8,  0,  0, P_T |P_M |P_L,
"cruise 3",         9,   4, 1500, 455, 14,  10,  0,  7,  9,  0,  0, P_T |P_M |P_L,
"cruise 4",         8,   3, 1750, 490,  6,  12,  0,  8, 10,  0,  0, P_T |P_M |P_L,
"cruise 5",         7,   2, 2000, 550,  2,  14,  0,  9, 11,  0,  0, P_T |P_M |P_L,
"hyp cruise 1",    16,   5, 6000, 600, 20,   4,  0,  8, 20,  0,  0, P_T |P_M |P_L,
"hyp cruise 2",    14,   4, 7100, 625, 12,  12,  0,  9, 30,  0,  0, P_T |P_M |P_L,
"hyp cruise 3",    12,   3, 8200, 650,  4,  20,  0, 10, 40,  0,  0, P_T |P_M |P_L,

/* short-range sams */
/*   name         lcm  hcm   $$$ tech acc load att def rng mil gas  flags */
"sr sam 1",        14,   8,  300, 230, 10,   4, 28,  1,  2,  0,  0, P_F |P_M |P_L,
"sr sam 2",        13,   7,  425, 260,  9,   4, 34,  1,  2,  0,  0, P_F |P_M |P_L,
"sr sam 3",        12,   7,  550, 290,  8,   4, 40,  1,  2,  0,  0, P_F |P_M |P_L,
"sr sam 4",        11,   6,  675, 320,  7,   4, 46,  1,  2,  0,  0, P_F |P_M |P_L,
"sr sam 5",        10,   6,  800, 350,  6,   4, 52,  1,  2,  0,  0, P_F |P_M |P_L,

#endif     

"",		    0,   0,    0,   0,  0,   0,  0,  0,  0,  0,  0, 0,
};

int	pln_maxno   = (sizeof(plchr) / sizeof(struct plchrstr)) - 1;
