/*
**  hunt.c - find spies and do something with them
**
**  8/23/98 Bill Frolik
*/

#include "misc.h"
#include "var.h"
#include "xy.h"
#include "spy.h"
#include "nsc.h"
#include "file.h"
#include "nat.h"
#include "loc.h"
#include "sect.h"
#include "ship.h"
#include "plane.h"
#include "item.h"
#include "nuke.h"
#include "news.h"
#include "path.h"
#include "deity.h"

extern double	searchcost;
extern double	killcost;
extern double	capturecost;

hunt()
{
    extern char *argp[];
    int mil,civ,spies,nspies,i,nsect,first;
    double odds,totalcost;
    struct nstr_item np,ns;
    struct natstr *natp;
    struct sctstr sect;
    struct shpstr ship;
    struct spystr spy;
    char action,*p,*loc;
    int loctype,loceff;
    natid locown;
    coord x,y,locx,locy;

    struct sls {
	int id,ship;
	coord x,y;
    } *spylist,*sl;
    
    /*
    **  Find out where to look for spies
    */
    p = getstarg(argp[1],"Search where (sector coord or ship list)? ");
    if (p) while (*p && *p<=' ') p++;
    if (!p || !*p) return RET_FAIL;
    if (*p=='@' || *p=='#' || index(p,',')) {
	loctype = EF_SECTOR;
	loc = (char *)&sect;
    } else {
	loctype = EF_SHIP;
	loc = (char *)&ship;
    }
    if (!snxtitem(&ns, loctype, p)) return RET_SYN;
    
    /*
    **  Snapshot a list of the requested spies
    */
    p = getstarg(argp[2],"Search for which spy ('*' for all)? ");
    if (!p || !*p) return RET_FAIL;
    if (!snxtitem(&np,EF_SPY,p)) return RET_SYN;
    for (spies=0; nxtitem(&np, (char *)&spy); spies++);
    spylist = (struct sls *)malloc(sizeof(struct sls) * (spies+4));
    snxtitem(&np,EF_SPY,p);
    for (spies=0; nxtitem(&np, (char *)&spy);) {
	if (loctype==EF_SHIP && spy.spy_ship<0) continue;
	if (loctype==EF_SECTOR && spy.spy_ship>=0) continue;
	if (spy.spy_own && spy.spy_own!=cnum) {
	    sl = &spylist[spies++];
	    sl->id = spy.spy_uid;
	    sl->x = spy.spy_x;
	    sl->y = spy.spy_y;
	    sl->ship = spy.spy_ship;
	    dpr(fmt("%s added to list",spyas(&spy,LOS)));
	}
    }

    /*
    **  Find out what to do with them if any are located
    */
    p = getstarg(argp[3],"Take what action (none, capture, kill)? ");
    if (!p || !*p || *p=='n') action = 'n';
    else if (*p=='c') action = 'c';
    else if (*p=='k') action = 'k';
    else return RET_SYN;

    natp = getnatp(cnum);
    totalcost = 0.0;

    for (nsect=nspies=0; nxtitem(&ns,loc); ) {
	locown = ((struct sctstr *)loc)->sct_own;
	loceff = ((struct sctstr *)loc)->sct_effic;
	locx = ((struct sctstr *)loc)->sct_x;
	locy = ((struct sctstr *)loc)->sct_y;
	if (!god && locown!=cnum) continue;
	mil = getvar(V_MILIT,loc,loctype);
	if (!mil) continue;
	mil = (mil+1)/2; /* only half the mil take part in the search */
	totalcost += ((double)mil*searchcost);
	civ = getvar(V_CIVIL,loc,loctype);
	pr(fmt("%d mil searching ",mil));
	if (civ) pr(fmt("among %d civ%s ",civ,splur(civ)));
	pr(fmt("%s\n",locas(loc,loctype,LPS)));
	for (first=i=0; i<spies; i++) {
	    if ((loctype==EF_SHIP && 
		 (spylist[i].ship==ns.cur)) ||
		(loctype==EF_SECTOR && 
		 (spylist[i].x==locx && spylist[i].y==locy))) {
		getspy(spylist[i].id,&spy);
		odds = (15.0*(double)mil) 
		    / ((double)civ+1.0)
			* ((double)loceff/100.0)
			    * (double)(natp->nat_level[NAT_ELEV]+20)
				/ (double)(spy.spy_edu+20);
		dpr(fmt("%s, %.2f chance of being found",spyas(&spy,LOS),odds));
		if (spy.spy_ship < 0) {
		    wu(0, spy.spy_own,
		       fmt("%s reports %d mil searching %s",
			   spyas(&spy,0),
			   round(estimate(mil,errormargin(&spy)),10),
			   locasc(loc,loctype,LOPS,spy.spy_own)));
		}
		if ( chance(odds) ) {
		    if (!first++) 
			pr(fmt("Troops have located in %s:\n",
			       xyas(locx,locy,cnum)));
		    pr(fmt("  %s ",spyas(&spy,LO)));
		    nspies++;
		    switch (action) {
		    case 'c': 
			odds = (double)mil/150.0
			    * ((double)loceff/100.0);
			if (odds > 0.95) odds = 0.95;
			dpr(fmt("capture success odds: %.2f",odds));
			if (chance(1.0-odds)) {
			    pr("escapes");
			    fleespy(&spy);
			} else {
			    pr("captured");
			    lostcontact(&spy);
			    if (chance(0.1)) {
				pr(", commits suicide");
				spy.spy_own = 0;
				spy.spy_effic = 0;
				dolcost += killcost;
			    } else {
				nreport(cnum,N_CAPTURE_SPY,spy.spy_own,1);
				spy.spy_own = cnum;
				spy.spy_effic = 10;
				dolcost += capturecost;
			    }
			    spy.spy_mobil = 0;
			    spy.spy_hap = 0;
			    putspy(spy.spy_uid,&spy);
			}
			break;
		    case 'k':
			pr("spotted\n\t");
			if (chance(0.5)) pr(">bang!<  ");
			else pr(">bbrrr-a-a-aap!<  ");
			odds = (double)mil/75.0
			    * ((double)loceff/100.0);
			if (odds > 0.95) odds = 0.95;
			dpr(fmt("kill success odds: %.2f\n",odds));
			if (chance(1.0-odds)) {
			    pr("... missed");
			    fleespy(&spy);
			} else {
			    pr("... and terminated");
			    nreport(cnum,N_KILL_SPY,spy.spy_own,1);
			    lostcontact(&spy);
			    spy.spy_own = 0;
			    spy.spy_effic = 0;
			    putspy(spy.spy_uid,&spy);
			    totalcost += killcost;
			}
			break;
		    }
		    pr("\n");
		}
	    }
	}
    }
    
    dolcost += (int)(totalcost+0.5);
    free(spylist);
    summarize(argp[1],nspies,"spy");
    return RET_OK;
}

lostcontact(spy)
struct spystr *spy;
{
    wu(0, spy->spy_own, 
       fmt("Lost contact with %s in %s",
	   spyas(spy,0),
	   xyas(spy->spy_x,spy->spy_y,spy->spy_own)));
}

/*
**  fleespy - move a spy to a more favorable sector
*/

fleespy(spy)
struct spystr *spy;
{
    char *p;
    struct sctstr sect;
    int i,j,k,mil,nsects;
    coord x,y;
    struct chstr {
	int dir,weight;
    } choice[7];
    
    if (spy->spy_ship >= 0 || !spy->spy_own || spy->spy_effic < 5) return;

    for (nsects=i=0; i<=6; i++) {
	x = spy->spy_x + diroff[i][0];
	y = spy->spy_y + diroff[i][1];
	if (!getsect(x, y, &sect)) continue;
	if (sect.sct_type==SCT_WATER || sect.sct_type==SCT_WASTE) continue;
	mil = getvar(V_MILIT,(char *)&sect,EF_SECTOR);
	if (mil > 999) mil = 999;
	if (sect.sct_own == cnum) j = 2000;	/* 1st choice: homeland */
	else if (sect.sct_own) j = 1000;	/* 2nd choice: non-POGO land */
	else j = 0;				/* 3rd choice: POGO land */
	choice[nsects].weight = j + mil;
	choice[nsects].dir = i;
	nsects++;
    }

    if (!nsects) return;

    for (i=0; i<nsects-1; i++)
	for (j=i+1; j<nsects; j++)
	    if (choice[i].weight < choice[j].weight) {
		k = choice[i].weight; 
		choice[i].weight = choice[j].weight;
		choice[j].weight = k;
		k = choice[i].dir; 
		choice[i].dir = choice[j].dir;
		choice[j].dir = k;
	    }
    
    if (nsects<3) i = 0;   /* If only 2 choices, always pick the first one; */
    else i = random() % 1; /* otherwise pick at random one of the top two.  */

    x = diroff[i][0];
    y = diroff[i][1];
    if ((x|y) != 0) {
	spy->spy_x = xnorm(spy->spy_x + x);
	spy->spy_y = ynorm(spy->spy_y + y);
	p = fmt("%s relocating to %s",spyas(&spy,0),
		xyas(spy->spy_x,spy->spy_y,spy->spy_own));
	if (cnum==spy->spy_own) pr(fmt("%s\n",p));
	else wu(0,spy->spy_own,p);
	putspy(spy->spy_uid,spy);
    }
}
