#ifndef lint
static char *RCSid = "$Header: orde.c,v 1.1 89/12/28 17:36:00 bill Exp $";
#endif

/*
 * orde.c
 *
 * turn on/off autonavigation
 *
 * from PSL Empire, 1985
 */

#include <time.h>
#include <ctype.h>
#include "misc.h"
#include "var.h"
#include "ship.h"
#include "sect.h"
#include "news.h"
#include "xy.h"
#include "nsc.h"
#include "nat.h"
#include "path.h"
#include "deity.h"
#include "file.h"
#include "item.h"

extern int etu_per_update;
extern int adj_update;
extern int s_p_etu;
extern int ship_mob_scale;

/*
**  Command syntax:
**
**  ORDER <ship>				  Show orders
**
**  ORDER <ship> e[ta]                            Show estimated arrival time
**
**  ORDER <ship> c[ancel]			  Cancel orders
**
**  ORDER <ship> s[top]				  Suspend orders
**
**  ORDER <ship> r[esume]			  Resume orders
**
**  ORDER <ship> <coord>			  Goto dest
**
*/

double basemobcost();

orde()
{
    extern char *argp[];
    int nships,orders,turnon,turnoff;
    int i,nupdates,showeta;
    double mobcost,mobil;
    struct nstr_item nb;
    struct shpstr ship;
    struct ichrstr *ip;
    struct mchrstr *mcp;
    int	secs_per_update;
    time_t now,	upd_time, next_update;
    coord px,py;
    char *p,*c;
    
    (void) time(&now);
    secs_per_update = etu_per_update * s_p_etu;
    if (!snxtitem(&nb, EF_SHIP, argp[1])) return RET_SYN;
    
    orders = showeta = turnon = turnoff = 0;
    if (argp[2]) switch (*argp[2]) {
    case 'e': orders=ORDER_ETA;    break;
    case 'c': orders=ORDER_CANCEL; turnoff=AN_AUTONAV+AN_STANDBY; break;
    case 's': orders=ORDER_STOP;   turnon=AN_STANDBY; break;
    case 'r': orders=ORDER_RESUME; turnoff=AN_STANDBY; break;
    }
    
    if (!orders) {
	if (argp[2] && *argp[2]) {
	    if ((p=getstarg(argp[2],""))==0 || *p==0) return RET_SYN;
	    if (!sarg_xy(p, &px, &py)) return RET_SYN;
	    orders = ORDER_P;
	}
    }
    
    switch (orders) {
    case ORDER_P:
	turnon = AN_AUTONAV;
	turnoff = AN_STANDBY;
	break;
    case ORDER_ETA:
	orders = 0;
	showeta++;
	pr("Estimated Arrival Times\n");
	break;
    }

    for (nships=0; nxtitem(&nb, (char *)(&ship)); ) {
	
	if (!owner || ship.shp_own==0) continue;

	if (ship.shp_type < 0 || ship.shp_type > shp_maxno) {
	    pr(fmt("bad ship type %d (#%d)\n",ship.shp_type,nb.cur));
	    continue;
	}
	
	if (!orders && (ship.shp_autonav & AN_AUTONAV)) {

	    if (!nships) { /* 1st ship, print banner */
		if (god) pr("own ");
		pr("veh#    vehicle type     x,y    f/c    dest");
		if (showeta) 
		    pr("    dist dir    eta");
		pr("\n");
	    }

	    if (god) pr(fmt("%3d ", ship.shp_own));
	    pr(fmt("%4d ", nb.cur));
	    pr(fmt("%-16.16s ", mchr[ship.shp_type].m_name));
	    pr(fmt("%s ",xyfmt("%4d,%-4d", ship.shp_x, ship.shp_y, cnum)));
	    pr(fmt("%1c %1c ", ship.shp_fleet,ship.shp_convoy));
	    pr(fmt(" %s ",xyfmt("%4d,%-4d",
				ship.shp_destx,ship.shp_desty,cnum)));
	    
	    if (showeta) {

		if (ship.shp_aboard >= 0) {
		    pr(" aboard transport, eta unknown\n");
		} else {

		    /*
		    **  Calculate route to destination, then simulate
		    **  a bunch of updates to figure out when ship will
		    **  arrive.
		    */

		    c = (char *)bestpath(ship.shp_x, ship.shp_y,
					 ship.shp_destx, ship.shp_desty,
					 mchr[ship.shp_type].m_terrain);

		    dpr("\npath=%s\n",c);
		    if (!c || !*c)      pr("no route possible\n");
		    else if (*c == 'h') pr("has arrived\n");
		    else if (*c == '?') pr("route too long\n");
		    else {
			i = strlen(c); /* distance to destination */
			pr(fmt(" %-3d  %c ",i,*c));
			mcp = &mchr[ship.shp_type];
			mobcost = basemobcost(&ship);
			upd_time = now + adj_update;
			next_update = now + secs_per_update
			    - (upd_time % secs_per_update);
			mobil = ship.shp_mobil;

			for (nupdates=1; i; ) {
			    if (mobil > 0) { /* simulate one sector move */
				mobil -= mobcost;
				i--;
			    } else { /* simulate mobility recharge */
				mobil += (ship_mob_scale * etu_per_update);
				upd_time = next_update + adj_update;
				next_update += secs_per_update
				    - (upd_time % secs_per_update);
				nupdates++;
			    }
			}

			pr(fmt(" %16.16s\n",ctime(&next_update)));
		    }
		}

	    } else {
		pr("  ");
		if (ship.shp_autonav & AN_STANDBY) pr("sby");
		pr("\n");
	    }
	    
	    nships++;
	}

	else {
	    
	    switch (orders) {
	    case ORDER_P:
		ship.shp_destx = px;
		ship.shp_desty = py;
	    case ORDER_RESUME:
	    case ORDER_CANCEL:
	    case ORDER_STOP:
		ship.shp_autonav &= ~turnoff;
		ship.shp_autonav |= turnon;
		putship(ship.shp_uid, &ship);
		
	    }
	}
    }
	
    if (!orders) {
	if (nships == 0) {
	    pr(fmt("%s: No ship(s)\n", (argp[1] == 0) ? "" : argp[1]));
	    return RET_FAIL;
	}
	else pr(fmt("%d ship(s)\n",nships));
	
    }
    
    return RET_OK;
}
