/*
**  recr.c
**
**  Recruit spies. 
**
**  7/91 Bill Frolik
*/

#include "misc.h"
#include "var.h"
#include "sect.h"
#include "spy.h"
#include "nat.h"
#include "xy.h"
#include "nsc.h"
#include "deity.h"
#include "file.h"

extern double spy_cost_multiplier;
extern double spy_tech_multiplier;

static struct spchrstr *askspytype();
static int recruit_spy();
static int cash;

/*
** recruit <SECTS> <TYPE>
*/

recr()
{
#ifdef SPIES
    extern char *argp[];
    extern double dolcost;
    struct sctstr sect;
    struct nstr_sect nstr;
    struct natstr *natp;
    int	rqtech,tlev,n;
    struct spchrstr *scp;
    int	vec[I_MAX+1];
    natid forwhom;
    char *p;
    int	gotsect = 0;
    
    natp = getnatp(cnum);
    cash = natp->nat_money;
    if (!snxtsct(&nstr, argp[1])) {
	pr("Bad sector specification.\n");
	return RET_SYN;
    }

    tlev = (int) natp->nat_level[NAT_TLEV];
    if (!(scp=askspytype(argp[2],tlev))) return RET_SYN;
    rqtech = (int)((double)scp->sp_tech*spy_tech_multiplier);
    if (rqtech > tlev) {
	pr(fmt("Required tech is %d\n",rqtech));
	return RET_FAIL;
    }

    forwhom = cnum;
    if (god) {
	if ((forwhom=natarg(argp[3],"Recruit for whom? ")) <= 0) 
	    forwhom = 0;
    }

    while (nxtsct(&nstr, &sect)) {
	gotsect++;
	if (!owner) continue;
	if (sect.sct_effic < 60 && !god) continue;
	getvec(VT_ITEM, vec, (char *)&sect, EF_SECTOR);
	if (recruit_spy(&sect, scp, vec, tlev, forwhom))
	    putvec(VT_ITEM, vec, (char *)&sect, EF_SECTOR);
	putsect(&sect);
    }
    if (!gotsect) pr("Bad sector specification.\n");
    return RET_OK;
#endif
}

#ifdef SPIES
static struct spchrstr *askspytype(s,tlev)
char *s;
{
    char *p;
    int type,n;
    struct spchrstr *scp;
    type = spy_maxno+1;
    while (type > spy_maxno) {
	p = getstarg(s,"Spy type? ");
	if (!p || !*p) return 0;
	s = (char *)0;
	n = strlen(p);
	for (scp = spchr, type=0; type <= spy_maxno; type++, scp++) {
	    if (scp->sp_tech > tlev && !god) continue;
	    if (!strncmp(p, scp->sp_name, n)) break;
	}
	if (type > spy_maxno) {
	    pr("Possible spy types are:\n");
	    show_spy_build(tlev);
	}
    }
    return(scp);
}

static int recruit_spy(sp, spyp, vec, forwhom)
register struct sctstr *sp;
register struct spchrstr *spyp;
register int *vec;
natid forwhom;
{
    struct spystr spy;
    int	cost;
    int	n;
    struct natstr *natp;
    struct nstr_item nstr;

    if (!god) {
	if (sp->sct_type != SCT_CAPIT) return 0;
	if (vec[I_LCM] < spyp->sp_lcm || 
	    vec[I_BAR] < spyp->sp_bar ||
	    vec[I_HCM] < spyp->sp_hcm) {
	    pr(fmt("Not enough materials in %s\n",
		   xyas(sp->sct_x, sp->sct_y, cnum)));
	    return 0;
	}
	cost = (int)((double)spyp->sp_cost*spy_cost_multiplier);
	if (cash < cost) {
	    pr(fmt("Not enough money left to recruit a %s\n", spyp->sp_name));
	    return 0;
	}
	if (!vec[I_MILIT]) {
	    pr(fmt("No military in %s\n",xyas(sp->sct_x, sp->sct_y, cnum)));
	    return 0;
	}
	dolcost += cost;
	cash -= cost;
    }
    
    natp = getnatp(cnum);
    snxtitem_all(&nstr, EF_SPY);
    while (nxtitem(&nstr, (char *)&spy))
	if (spy.spy_own == 0) break;
    spy.spy_x = sp->sct_x;
    spy.spy_y = sp->sct_y;
    spy.spy_own = (spy.spy_oldown = god) ? forwhom : sp->sct_own;
    spy.spy_type = spyp - spchr;
    spy.spy_uid = nstr.cur;
    spy.spy_age = 0;
    spy.spy_mobil = 0;
    spy.spy_effic = 20;
    spy.spy_tech = natp->nat_level[NAT_TLEV];
    spy.spy_edu = natp->nat_level[NAT_ELEV];
    spy.spy_hap = natp->nat_level[NAT_HLEV];
    spy.spy_ship = -1;
    spy.spy_sell = 0;
    spy.spy_plntech = 0;
    spy.spy_flags = 0;
    spy.spy_nv = 0;
    *spy.spy_name = '\0';
    putspy(nstr.cur, &spy);
    pr(fmt("%s #%d recruited ", spyp->sp_name, nstr.cur));
    if (god && forwhom!=cnum) pr(fmt("for %s ",cname(forwhom)));
    pr(fmt("in sector %s\n",xyas(sp->sct_x, sp->sct_y, cnum)));
    if (!god) {
	vec[I_LCM] -= spyp->sp_lcm;
	vec[I_HCM] -= spyp->sp_hcm;
	vec[I_BAR] -= spyp->sp_bar;
	vec[I_MILIT]--;
    }
    return 1;
}
#endif
