/*
** spy.c
*/

#include "misc.h"
#include "var.h"
#include "xy.h"
#include "spy.h"
#include "nsc.h"
#include "file.h"
#include "nat.h"
#include "sect.h"
#include "ship.h"
#include "plane.h"
#include "item.h"
#include "nuke.h"
#include "news.h"
#include "path.h"
#include "deity.h"

#ifdef SPIES

/*
**  spy <id> <cmd>
**
**  8/91 Bill Frolik
*/

int spyreport();
int spyreportcom();
int spyreportlev();
int spyreportcom();
int spyreportcen();
int spyreportplanes();
int spyreportvc();
int spyreportvc();
int spyreportvc();
int spyreportvc();
int spyrecon();
int spybomb();
int spyset();
int spytake();
int spydrop();
int spykill();
int spyboard();
int spyleave();
int spymove();
int spygive();
int spylook();
int spyretire();
int spysteal();

struct {
    char *keyword;
    int (*function)();
    int vc;
    int eftype;
    int difficulty;
} spycmd[] = {
    /* THIS LIST MUST BE IN ALPHABETICAL ORDER */
    "balloons",		spyreportvc,	CLASS_BALLOON,	0,		25,
    "board",		spyboard,	0,		0,		40,
    "bomb",		spybomb,	0,		0,		-1,
    "cargo",		spyreportcom,	0,		EF_SHIP,	55,
    "census",		spyreportcen,	0,		0,		 5,
    "commodity",	spyreportcom,	0,		EF_SECTOR,	30,
    "drop",		spydrop,	0,		0,		 1,
    "efficiency",       spyset,         0,              0,              -2,
    "give",		spygive,	0,		0,		 2,
    "kill",		spykill,	0,		0,		20,
    "leave",		spyleave,	0,		0,		10,
    "level",		spyreportlev,	0,		EF_SECTOR,	45,
    "look",		spylook,	0,		0,		 0,
    "mobility",         spyset,         0,              0,              -2,
    "move",		spymove,	0,		0,		40,
    "names",		0,		0,		-1,		 0,
    "planes",		spyreportplanes,0,		0,		35,
    "recon",		spyrecon,	0,		0,		15,
    "report",		spyreport,	0,		0,		 0,
    "retire",		spyretire,	0,		0,		 0,
    "ships",		spyreportvc,	CLASS_SHIP,	0,		15,
    "steal",		spysteal,	0,		0,		95,
    "take",		spytake,	0,		0,		-1,
    "tanks",		spyreportvc,	CLASS_TANK,	0,		30,
    "trains",		spyreportvc,	CLASS_TRAIN,	0,		15,
    "",			0,		0,		0,		 0,
};

spy()
{
    extern char *argp[];
    int i,nsp,j,cmd,help;
    int vec[I_MAX+1];
    struct nstr_item np;
    struct sctstr sect;
    struct shpstr ship;
    struct plnstr plane;
    char *p,*loc,buf[32];
    int loctype;
    coord x,y,nx,ny;
    struct spystr spy;

    if (!snxtitem(&np, EF_SPY, argp[1])) return RET_SYN;

    cmd = -1;
    if (argp[2]) {
	for (help=cmd=0; *spycmd[cmd].keyword; cmd++)
	    if (!strncmp(argp[2],spycmd[cmd].keyword,strlen(argp[2]))) break;
	if (!*spycmd[cmd].keyword) {
	    pr(fmt("'%s' is not a valid option\n",argp[2]));
	    help++;
	} else {
	    if (!strncmp(argp[2],spycmd[cmd+1].keyword,strlen(argp[2]))) {
		pr(fmt("'%s' is not unique\n",argp[2]));
		help++;
	    }
	}
	if (help) {
	    pr("Try one of these instead:\n");
	    for (i=0; i<4; i++) pr("  dif  task        "); pr("\n");
	    for (i=0; i<4; i++) pr("  ---  ----------- "); pr("\n");
	    for (i=cmd=0; *spycmd[cmd].keyword; cmd++) {
		if (spycmd[cmd].difficulty == -2 && !god) continue;
		sprintf(buf,"  %3d  %-11.11s ",
			spycmd[cmd].difficulty,spycmd[cmd].keyword);
		for (p=buf; *p; p++) if (*p == '*') *p = ' ';
		pr(buf);
		if (((i++) % 4) == 3) pr("\n");
	    }
	    if ((i % 3)) pr("\n");
	    pr("(-1) difficulty varies with commodity\n");
	    if (god) pr("(-2) deity-only command\n");
	    return RET_SYN;
	}
    }

    if (cmd>=0 && spycmd[cmd].eftype<0) cmd = -1;

    for (nsp=0; nxtitem(&np, (char *)&spy);) {
	if ((!god && spy.spy_own!=cnum) || !spy.spy_own) continue;
	if (!getsect(spy.spy_x,spy.spy_y,&sect)) continue;
	if (cmd < 0)
	    showspy(&spy,(nsp++ == 0),0);
	else {
	    loctype = EF_SECTOR; 
	    loc = (char *)&sect;
	    if (spy.spy_ship >= 0) {
		if (!getship(spy.spy_ship,&ship)) continue;
		loctype = EF_SHIP; 
		loc = (char *)&ship;
	    }
	    j = (spycmd[cmd].function)
		(&spy, argp, loc, 
		 spycmd[cmd].eftype ? spycmd[cmd].eftype : loctype, 
		 spycmd[cmd].vc, spycmd[cmd].difficulty);
	    if (j) {
/*		pr("\n"); */
		nsp++;
	    }
	}
    }
    writebigmap();
/*  return summarize(argp[1],nsp,"spy"); */
    return (nsp ? RET_OK : RET_FAIL);
}

showspy(spy,header,plusorminus)
struct spystr *spy;
int header,plusorminus;
{
    natid host;
    struct sctstr sect;
    struct shpstr ship;

    if (header) {
	prest(plusorminus);
	if (god) pr("own ");
	pr("spy#   spy type   name        x,y   abr  eff  mu ");
	pr("tech edu hap age acc plt ship\n");
    }
    if (god) pr(fmt("%3d ", spy->spy_own));

    host = 0;
    if (spy->spy_ship >= 0) {
	if (getship(spy->spy_ship,&ship)) 
	    host = ship.shp_own;
    } else {
	if (getsect(spy->spy_x,spy->spy_y,&sect))
	    host = sect.sct_own;
    }

    pr(fmt("%4d %-10.10s %-10.10s %s ",
	   spy->spy_uid,spchr[spy->spy_type].sp_name,
	   spy->spy_name,xyfmt("%4d,%-4d",spy->spy_x,spy->spy_y, cnum)));
    pr(fmt("%c  %3d%% %3d %4d %3d %3d %3d %3d %3d ",
	   host==spy->spy_own ? ' ' : '*',
	   limestimate(spy->spy_effic,plusorminus,0,100), 
	   estimate(spy->spy_mobil,plusorminus),
	   estimate(spy->spy_tech,plusorminus),
	   estimate(spy->spy_edu,plusorminus), 
	   estimate(spy->spy_hap,plusorminus),
	   estimate(spy->spy_age,plusorminus),
	   limestimate(100-errormargin(spy),plusorminus,0,100),
	   estimate(spy->spy_plntech,plusorminus) ));
    if (spy->spy_ship >= 0) 
	pr(fmt("%4d", spy->spy_ship));
    else 
	pr("    ");
    pr("\n");
}

#endif


/*
 * oldspy.c
 *
 * spy on your neighbors
 *
 * from PSL Empire, 1985
 * and Dave Pare, 1986
 */

/*
 * format:  spy <SECTS>
 *
 */
static int nsects;
static int spyline();
static int insert();
static int check();

snoop()
{
	extern	char *argp[];
	int     caught;
	natid	own;
	int     relat;
	coord	x, y;
	coord	nx, ny;
	int     military;
	int     savemil;
	int     btucost;
	int     i;
	coord   *table;		    /* sectors already seen */
	int	t_len = 0;
	int	vec[I_MAX+1];
	int	dvec[I_MAX+1];
	struct nstr_sect nstr;
	struct natstr *natp;
	struct sctstr from;
	struct sctstr dsect;

	/*
	 * first arg should be the range of sectors
	 */
	if (!snxtsct(&nstr, argp[1]))
		return RET_SYN;
	nsects = (nstr.range.width+1) * nstr.range.height / 2;
	btucost = (nsects / 40) + 1;
	natp = getnatp(cnum);
	if (natp->nat_btu < btucost) {
		pr("You don't have the BTU's for spying on that scale!\n");
		return RET_FAIL;
	}
	/*
	 * set up all the goodies we need later
	 * 6 = neighbors, 2 = x,y
	 */
	table = (coord *) malloc((nsects + 1) * 6 * 2 * sizeof(coord));
	bzero((char *)table, (nsects + 1) * 6 * 2 * sizeof(coord));
	pr("SPY report\n");
	prdate();
	pr("   sect   type own  eff  civ  mil  shl  gun  iron  pet  food\n");
	while (nxtsct(&nstr, &from)) {
		if (!owner && !god)
			continue;
		getvec(VT_ITEM, vec, (char *) &from, EF_SECTOR);
		if ((military = vec[I_MILIT]) == 0)
			continue;
		x = from.sct_x;
		y = from.sct_y;
		savemil = military;
		/*
		 * check the neighboring sectors.
		 */
		for (i = 1; i <= 6; i++) {
			if (military == 0)
				break;
			nx = x + diroff[i][0];
			ny = y + diroff[i][1];
			/*
			 * if we've already seen the
			 * sector, don't bother checking it
			 * out.
			 */
			if (check(table, &t_len, nx, ny)) {
				continue;
			}
			getsect(nx, ny, &dsect);
			getvec(VT_ITEM, dvec, (char *) &dsect, EF_SECTOR);
			if (owner || (!dvec[I_MILIT] && !dvec[I_CIVIL])) {
				/*
				 * tell the table we've
				 * seen the sector
				 */
				insert(table, &t_len, nx, ny);
				continue;
			}
			/*
			 * roll 1d200 to see if the
			 * patrolling military get the spy.
			 * If the roll is less than the
			 * military in the sector, BANG!!
			 */
			caught = chance((double) dvec[I_MILIT] / 200.0);
			own = dsect.sct_own;
			/*
			 * get the nation status out of the
			 * two bits allotted each country
			 * for the relations
			 */
			relat = getrel(natp, dsect.sct_own);
			if (relat == NEUTRAL && caught) {
				/*
				 * neutral spy-ee
				 */
				pr(fmt("Spy deported from %s\n", 
					xyas(nx, ny, cnum)));
				wu(0, own, fmt("%s (#%d) spy deported from %s",
					cname(cnum), cnum, xyas(nx, ny, own)));
				continue;
			} else if ((relat == AT_WAR || relat == HOSTILE)
				   && caught) {
				/*
				 * at-war with spy-ee
				 */
				pr(fmt("BANG!! A spy was shot in %s\n",
				       xyas(nx, ny, cnum)));
				military--;
				wu(0, own, fmt("%s (#%d) spy caught in %s",
					cname(cnum), cnum, xyas(nx, ny, own)));
				nreport(cnum, N_SPY_SHOT, own, 1);
				continue;
			}
			insert(table, &t_len, nx, ny);
			spyline(&dsect);
			setbigmap(dsect.sct_x, dsect.sct_y,
				dchr[dsect.sct_type].d_mnem);
		}
		writebigmap();
		/*
		 * subtract any military if necessary from
		 * BANG!! (I like that a lot)
		 */
		if (savemil != military) {
			vec[I_MILIT] = military;
			putvec (VT_ITEM, vec, (char *) &from, EF_SECTOR);
			putsect(&from);
		}
	}
	NAT_DELTA(nat_btu, cnum, -btucost);
	free((char *)table);
	return RET_OK;
}


/*
 * just a big printf.
 */
static
spyline(sp)
	struct sctstr *sp;
{
	char   *fmt();
	int vec[I_MAX+1];

	getvec(VT_ITEM, vec, (char *) sp, EF_SECTOR);
	pr(xyfmt("%4d,%-4d",sp->sct_x, sp->sct_y, cnum));
	pr(fmt("   %c  %3d  %3d %4d  %3d  %3d  %3d   %3d  %3d  %4d\n",
	       dchr[sp->sct_type].d_mnem, sp->sct_own,
	       round((int)sp->sct_effic, 10),
	       round(vec[I_CIVIL], 10), round(vec[I_MILIT], 10),
	       round(vec[I_SHELL], 10), round(vec[I_GUN], 10),
	       round(vec[I_IRON], 10), round(vec[I_PETROL], 10),
	       round(vec[I_FOOD], 10)));
}


/*
 * insert a key into the table.
 */
static
insert(table, len, x, y)
	coord  *table;
	int    *len;
	coord     x;
	coord     y;
{
	if (!check(table, len, x, y)) {
	   table[(*len)++] = x;
	   table[(*len)++] = y;
	}
}
/*
 * see if a key is in the bitmask table
 */
static int
check(table, len, x, y)
	coord    *table;
	int    *len;
	coord     x;
	coord     y;
{
	int     i;

	for (i = 0; i < *len; i += 2)
	   if (table[i] == x && table[i + 1] == y)
	      return 1;
	return 0;
}
