#ifndef lint
static char *RCSid = "$Header: aircombat.c,v 1.1 89/12/14 13:28:06 jay Exp $";
#endif

/*
 * aircombat.c
 *
 * Deal with air to air combat
 *
 * Dave Pare, 1986
 */

#include "misc.h"
#include "var.h"
#include "news.h"
#include "ship.h"
#include "sect.h"
#include "nuke.h"
#include "plane.h"
#include "nat.h"
#include "file.h"
#include "xy.h"
#include "nsc.h"
#include "path.h"
#include "loc.h"

static	getilist();

extern double flak_factor;	/* flak intensity multplier */
extern int    flak_base;        /* flak random base */
extern int    flak_range;       /* flak random spread */

#define	DOG_MAX		15
#define FLAK_MAX	15

		/*       -7    -6    -5    -4    -3    -2    -1    0 */
float dogtable[16] = {	0.10, 0.15, 0.28, 0.45, 0.60, 0.75, 0.82, 1.00,
			1.10, 1.25, 1.60, 1.90, 2.25, 3.00, 4.15, 5.50, };
float flaktable[16] = {	0.30, 0.30, 0.35, 0.40, 0.40, 0.45, 0.50, 0.50,
			0.50, 0.55, 0.60, 0.65, 0.70, 0.75, 0.80, 0.85, };

visiblevehicles_xy(vehlist,x,y)
int *vehlist;
coord x,y;
{
    int nvehs;
    struct nstr_item ni;
    struct shpstr ship;
    snxtitem_xy(&ni, EF_SHIP, x, y);
    for (nvehs=0; nxtitem(&ni, (char *)&ship);)
	if (ship.shp_own && (ship.shp_aboard<0)) {
	    if (vehlist) vehlist[nvehs] = ni.cur;
	    nvehs++;
	}
    if (vehlist) vehlist[nvehs] = -1;
    return nvehs;
}

visiblevehicles_all(vehlist)
int *vehlist;
{
    int nvehs;
    struct nstr_item ni;
    struct shpstr ship;
    snxtitem_all(&ni, EF_SHIP);
    for (nvehs=0; nxtitem(&ni, (char *)&ship);)
	if (ship.shp_own && (ship.shp_aboard<0)) {
	    if (vehlist) vehlist[nvehs] = ni.cur;
	    nvehs++;
	}
    if (vehlist) vehlist[nvehs] = -1;
    return nvehs;
}

ac_encounter(bomb_list,esc_list,x,y,path,mission_flags)
struct qelem *bomb_list,*esc_list;
coord x,y;
char *path;
int mission_flags;
{
    coord sx,sy;
    fly_flightpath(bomb_list,esc_list,x,y,path,mission_flags,&sx,&sy,0);
}

fly_flightpath(bomb_list,esc_list,x,y,path,mission_flags,sx,sy,cmeff)
struct qelem *bomb_list,*esc_list;
coord x,y,*sx,*sy;
char *path;
int mission_flags;
int cmeff; /* cruise missile (efficiency) */
{
    int	val,rel,dir,i;
    int	nats[MAXNOC];
    int	gotilist[MAXNOC];
    int	unfriendly[MAXNOC];
    int	overfly[MAXNOC];
    struct qelem ilist[MAXNOC];
    int	civ, mil;
    struct sctstr sect;
    double evade;
    double tlev;
    double diff;
    struct shpstr ship;
    struct nstr_item ni;
    natid cn;
    struct natstr *over;
    int *vehlist;

    bzero((char *)overfly, sizeof(overfly));
    bzero((char *)gotilist, sizeof(gotilist));
    bzero((char *)unfriendly, sizeof(unfriendly));

    for (cn=1; cn<MAXNOC; cn++) {
	rel = getrel(getnatp(cn), cnum);
	i = 0;
#ifdef ALLIEDENCOUNTERS
	if (rel == ALLIED) i++;
#endif
#ifdef NEUTRALENCOUNTERS
	if (rel == NEUTRAL) i++;
#endif
#ifdef HOSTILEENCOUNTERS
	if (rel == HOSTILE) i++;
#endif
#ifdef WARENCOUNTERS
	if (rel == AT_WAR) i++;
#endif
	if (i==0 || cnum==cn) continue;
	unfriendly[cn]++;
    }

    if (mission_flags & PM_R) {
	if (mission_flags & P_S)
	    sathead("\nSPY Plane report\n",mission_flags);
	else {
	    pr("\nReconnaissance report\n");
	    prdate();
	}
    }

    pln_removedupes(bomb_list, esc_list);

    vehlist = (int *)calloc(visiblevehicles_all((int *)0)+40,sizeof(int));
    
    /*
    **  Process the flight path
    */

    getsect(x, y, &sect);
    while ((dir = *path++) && !QEMPTY(bomb_list)) {

	val = chkdir(dir, DIR_STOP, DIR_LAST);

	if (val && cmeff && (cmeff < CRUISE_FAIL)) {
	    if (chance((double)(100-cmeff)/30.0)) {
		pr(fmt("Missile in-flight failure over %s\n",xyas(x,y,cnum)));
		val = 0;
	    }
	}

	if (val == 0) break;
	x = xnorm(x+diroff[val][0]);
	y = ynorm(y+diroff[val][1]);
	getsect(x, y, &sect);
	over = getnatp(sect.sct_own);

	visiblevehicles_xy(vehlist,x,y);

	if (unfriendly[sect.sct_own] && !cmeff)
	    if (mission_flags & P_X) {
		/* Stealth */
		tlev = (mission_flags >> 16);
		diff = tlev - over->nat_level[NAT_TLEV];
		evade = 0.88 + 0.12 * diff / tlev;
		if (!chance(evade))
		    ac_doflak(bomb_list, &sect);
	    } else
		ac_doflak(bomb_list, &sect);
	
	/* mission planes aborted due to flak -- don't send escorts */
	if (QEMPTY(bomb_list)) break;
	
	setbigmap(sect.sct_x,sect.sct_y,dchr[sect.sct_type].d_mnem);
	
	if (mission_flags & P_MS) { /* mine sweep */
	    ac_dosweep(bomb_list, &sect);
	}

	if (sect.sct_type != SCT_WATER) {
	    if (mission_flags & PM_R) {
		if (mission_flags & P_S) {
		    satdisp(&sect, (mission_flags&P_I)?10:50, mission_flags);
		} else {
		    /* This is borrowed from lookout */
		    if (sect.sct_own == cnum) pr("Your ");
		    else pr(fmt("%s (#%d) ", cname(sect.sct_own),
				sect.sct_own));
		    pr(dchr[sect.sct_type].d_name);
		    pr(fmt(" %d%% efficient ",
			   owner ? sect.sct_effic :
			   round((int)sect.sct_effic,25)));
		    civ = getvar(V_CIVIL, (char *)&sect, EF_SECTOR);
		    mil = getvar(V_MILIT, (char *)&sect, EF_SECTOR);
		    if (civ)
			pr(fmt("with %s%d civ ",
			       owner ? "" : "approx ",
			       owner? civ : round(civ, 25)));
		    if (mil)
			pr(fmt("with %s%d mil ",
			       owner ? "" : "approx ",
			       owner? mil : round(mil, 25)));
		    pr(fmt("@ %s\n", xyas(x, y, cnum)));
		}
	    } else {
		pr(fmt("flying over %s at %s\n",
		       dchr[sect.sct_type].d_name,
		       xyas(x, y, cnum)));
	    }
	}	    

	/*
	**  Recon ships/vehicles in the sector.
	**  Ship owner gets a telegram, but it doesn't make the news.
	**  4/12/91 bf@hp
	*/
	if (mission_flags & PM_R) {
	    int spotted[MAXNOC];
	    bzero((char *)spotted, sizeof(spotted));
	    for (i=0; vehlist[i]>=0; i++) {
		getship(vehlist[i],&ship);
		if (ship.shp_x!=x && ship.shp_y!=y) continue;
		if ( ship.shp_depth==0 || 
		    (ship.shp_depth==1 && (mission_flags & P_U)) )
		    pr(fmt("  %s %s #%d @ %s\n",
			   (ship.shp_own==cnum) ? "Your"
			   : cname(ship.shp_own),
			   mchr[ship.shp_type].m_name,
			   ship.shp_uid,
			   xyas(x,y,cnum)));
		if (!spotted[ship.shp_own] && ship.shp_own!=cnum) {
		    tlev = (mission_flags >> 16);
		    diff = tlev - over->nat_level[NAT_TLEV];
		    evade = (mission_flags & P_X)
			? 0.88 + 0.12 * diff / tlev
			    : 0.0;
		    if (chance(1.0-evade) &&
			!(issub(&ship) && ship.shp_depth>1)) {
			wu(0, ship.shp_own, 
			   fmt("%s #%d reports %s aircraft sighted in %s",
			       mchr[ship.shp_type].m_name,
			       ship.shp_uid,
			       cname(cnum), 
			       xyas(x, y, ship.shp_own)));
			spotted[ship.shp_own] = 1;
		    }
		}
	    }
	}
	
	if (sect.sct_own == 0 || sect.sct_own == cnum) {
	    ac_kamikaze(bomb_list, vehlist, x, y);
	    continue;
	}
	
	if (mission_flags & P_X) {
	    /* Stealth */
	    tlev = (mission_flags >> 16);
	    diff = tlev - over->nat_level[NAT_TLEV];
	    evade = 0.88 + 0.12 * diff / tlev;
	    if (chance(evade)) {
		ac_kamikaze(bomb_list, vehlist, x, y);
		continue;
	    }
	}
	
	if (unfriendly[sect.sct_own] && !gotilist[sect.sct_own]) {
	    getilist(&ilist[sect.sct_own], sect.sct_own);
	    gotilist[sect.sct_own]++;
	}

	if ((rel=getrel(over, cnum)) == ALLIED) {
	    ac_kamikaze(bomb_list, vehlist, x, y);
	    continue;
	}

	overfly[sect.sct_own]++;
	wu(0, sect.sct_own, 
	   fmt("%s %s spotted over %s",
	       cname(cnum),
	       cmeff ? "missiles" : "planes",
	       xyas(x, y, sect.sct_own)));
	
	if (rel == NEUTRAL) {
	    ac_kamikaze(bomb_list, vehlist, x, y);
	    continue;
	}
	
	i = ac_intercept(bomb_list, esc_list,
			 &ilist[sect.sct_own],
			 sect.sct_own, x, y);

	if (cmeff) {
	    cmeff -= i;
	    if (cmeff < 0) cmeff = 0;
	}

	ac_kamikaze(bomb_list, vehlist, x, y);

    }
    
    /*
    **  Pass back the coords where we finally ended up
    **  (used by cruise missiles).
    */

    *sx = x;
    *sy = y;

    writebigmap();
    free(vehlist);

    /*
    **  Now run through ships at the destination sector
    */

    if (QEMPTY(bomb_list) && QEMPTY(esc_list)) return;
    bzero((char *)nats, sizeof(nats));

    snxtitem_xy(&ni, EF_SHIP, x, y);
    while (nxtitem(&ni, (char *)&ship)) {
	if (ship.shp_depth==0) nats[ship.shp_own]++;
    }

    ac_shipflak(bomb_list, x, y);
    for (cn=1; cn<MAXNOC && !QEMPTY(bomb_list); cn++) {
	if (cnum == cn)
	    continue;
	if (!cmeff && overfly[cn]>0)
	    nreport(cnum, N_OVFLY_SECT, cn, overfly[cn]);
	if (nats[cn] == 0)
	    continue;
	wu(0, cn, 
	   fmt("%s %s spotted over ships in %s",
	       cname(cnum), 
	       cmeff ? "missiles" : "planes",
	       xyas(x, y, cn)));
	if (!unfriendly[cn])
	    continue;
	if (!gotilist[cn]) {
	    getilist(&ilist[cn], cn);
	    gotilist[cn]++;
	}
	if (!cmeff)
	    pr(fmt("Flying over %d #%d ships...\n", nats[cn], cn));
	/* This makes going for ships in harbors tough */
	ac_intercept(bomb_list, esc_list, &ilist[cn], cn, x, y);
    }
/*
    visiblevehicles_xy(vehlist,x,y);
    ac_kamikaze(bomb_list,vehlist,x,y); 
*/
}


/* ________________________________________________________________
**|                                                                |
**| ac_intercept - intercept flight over x,y                       |
**|                return damage inflicted on bombers              |
**|________________________________________________________________|
*/

ac_intercept(bomb_list, esc_list, def_list, def_own, x, y)
struct	qelem *bomb_list, *esc_list, *def_list;
natid	def_own;
coord	x,y;
{
    register struct plnstr *pp;
    register struct plchrstr *pcp;
    int	gas;
    struct	plist *ip;
    int	icount;
    struct	shpstr ship;
    struct	sctstr sect;
    struct	qelem *next;
    struct	qelem *qp;
    struct	qelem int_list;
    char	*ptr;
    int	type;
    int	att_count;
    
    icount = 0;
    att_count = 0;
    for (qp = bomb_list->q_forw; qp != bomb_list; qp = qp->q_forw)
	att_count++;
    for (qp = esc_list->q_forw; qp != esc_list; qp = qp->q_forw)
	att_count++;
    initque(&int_list);
    for (qp = def_list->q_forw; qp != def_list; qp = next) {
	next = qp->q_forw;
	ip = (struct plist *) qp;
	pp = &ip->plane;
	pcp = ip->pcp;
	if (pp->pln_range/2 < mapdist(x, y, pp->pln_x, pp->pln_y))
	    continue;
	if (pp->pln_ship >= 0) {
	    getship(pp->pln_ship, &ship);
	    type = EF_SHIP;
	    ptr = (char *) &ship;
	} else {
	    getsect(pp->pln_x, pp->pln_y, &sect);
	    type = EF_SECTOR;
	    ptr = (char *) &sect;
	}
	gas = getvar(V_PETROL, ptr, type);
	if (gas < (pcp->pl_fuel/2)) {
	    remque(qp);
	    free((char *)qp);
	    continue;
	}
	/* got one; delete from def_list, add to int_list */
	remque(qp);
	insque(qp, &int_list);
	pp->pln_mobil -= 10;
	putplane(pp->pln_uid, pp);
	putvar(V_PETROL, gas - pcp->pl_fuel/2, ptr, type);
	if (pp->pln_ship >= 0)
	    putship(pp->pln_ship, &ship);
	else
	    putsect(&sect);
	icount++;
	if (icount > att_count)
	    break;
    }
    if (icount == 0)
	return 0;
    pr(fmt("%d %s fighter%s rising to intercept!\n", icount,
	   cname(def_own), icount == 1 ? " is" : "s are"));
    ac_airtoair(esc_list, &int_list, def_own, x, y);
    return ac_airtoair(bomb_list, &int_list, def_own, x, y);
}


/* ________________________________________________________________
**|                                                                |
**| ac_airtoair - air-to-air combat                                |
**|               The interceptors aren't affected by efficiency   |
**|               loss; they always engage at 1:1 odds.            |
**|               Return total damage inflicted on attacker.       |
**|________________________________________________________________|
*/


ac_airtoair(att_list, int_list, def_own, x, y)
struct qelem *att_list;
struct qelem *int_list;
natid def_own;
coord x,y;
{
    register struct plist *attacker;
    register struct plist *interceptor;
    struct	qelem *att;
    struct	qelem *in;
    int	nplanes;
    int	adam;
    int	idam;
    int	more_att;
    int	more_int;
    int totaldam;
    char	*mesg;
    struct	qelem *att_next;
    struct	qelem *in_next;
    
    att = att_list->q_forw;
    in = int_list->q_forw;
    more_att = 1;
    more_int = 1;
    totaldam = 0;
    if (QEMPTY(att_list) || QEMPTY(int_list)) {
	more_att = 0;
	more_int = 0;
    }
    while (more_att || more_int) {
	in_next = in->q_forw;
	att_next = att->q_forw;
	attacker = (struct plist *) att;
	interceptor = (struct plist *) in;
	mesg = fmt("  %s %s %s",
		   interceptor->pcp->pl_name,
		   attacker->pcp->pl_flags & P_M
		   ? "attempting to intercept"
		   : "intercepts",
		   attacker->pcp->pl_name);
	pr(mesg); pr("\n");
	wu(0, def_own, mesg);
	adam = idam = 0;
	nplanes = attacker->plane.pln_effic;
	if (nplanes > interceptor->plane.pln_effic)
	    nplanes = interceptor->plane.pln_effic;
	if (interceptor->plane.pln_att) /* interceptor fires first */
	    adam = ac_dog(interceptor, attacker, nplanes);
	if (interceptor->pcp->pl_flags & P_M)
	    idam = 100; /* if interceptor is a missile, waste it */
	else if (attacker->plane.pln_att) /* else attacker retaliates */
	    idam = ac_dog(attacker, interceptor,
			  min(nplanes, attacker->plane.pln_effic));
	if (adam > 0)
	    ac_planedamage(attacker, interceptor->plane.pln_own,
			   adam, def_own, x, y);
	if (idam > 0)
	    ac_planedamage(interceptor, attacker->plane.pln_own,
			   idam, def_own, 0, 1); /* bogus x,y skips kamikaze */
	if ((att=att_next) == att_list) {
	    more_att = 0;
	    if (QEMPTY(att_list)) more_int = 0;
	    else att = att->q_forw;
	}
	if ((in=in_next) == int_list) {
	    more_int = 0;
	    if (QEMPTY(int_list)) more_att = 0;
	    else in = in->q_forw;
	}
	totaldam += adam;
    }
    return totaldam;
}

int ac_dog(ap, dp, nplanes)
register struct plist *ap;
register struct plist *dp;
int nplanes;
{
    int	n,diff;
    float mult;

    diff = (ap->plane.pln_att - dp->plane.pln_def) / 2;
    if ((ap->pcp->pl_flags & P_F) && ap->bombs == 0) diff++;
    if (ap->pcp->pl_flags & P_X) diff += 3;
    if ((dp->pcp->pl_flags & P_F) && dp->bombs == 0) diff--;
    if (dp->pcp->pl_flags & P_X) diff -= 3;
    if (diff > 8)
	mult = dogtable[DOG_MAX] * 1.33;
    else if (diff < -7)
	mult = dogtable[0] * 0.66;
    else {
	diff += 7;
	mult = dogtable[diff];
    }
    n = roll(33) * mult * nplanes / 100.0;
    if (n > 100)
	n = 100;
    return n;
}

/*
 * zap plane associated with plp.
 * Damaging country is "from", damage is "dam".
 * def_own is the country on the other side of the conflict from cnum.
 * The only time def_own != from is when the interceptor is getting damaged.
 *
 * NOTE: This routine removes the appropriate plane element from the
 * queue if it gets destroyed.  That means that the caller must assume
 * that the current queue pointer is invalid on return from the ac_planedamage
 * call.  (this has caused bugs in the past)
 */
ac_planedamage(plp, from, dam, def_own, x, y)
struct	plist *plp;
natid	from,def_own;
int	dam;
coord	x,y;
{
    register struct plnstr *pp;
    int	disp,eff;
    char *dmess,*mesg;
    int i,j,k,*vehlist;

    disp = 0;
    pp = &plp->plane;
    eff = pp->pln_effic;
    if (dam <= 0) return;
    dmess = "";
    eff -= dam;
    if (eff < 0) eff = 0;
    if ((plp->pcp->pl_flags & P_M) && (dam > 0)) {
	disp= (eff<CRUISE_KILL) ? 1 : 2;
	mesg = fmt("    %s (#%d) %s.",
		   plp->pcp->pl_name, pp->pln_uid,
		   (disp==1) ? "destroyed" : "damaged");
	pr(mesg); pr("\n");
	wu(0, def_own, mesg);
    } else if (eff < 10) {
	if (chance(0.5))
	    plp->kamikaze = -1;
	disp=1;
	dmess = " -- shot down";
    } else if (chance((dam*4)/100.0)) {
	disp=2;
	dmess = " -- aborted";
    }
    if ((plp->pcp->pl_flags & P_M) == 0) {
	mesg = fmt("    %s (#%d) takes %d%s.",
		   plp->pcp->pl_name, pp->pln_uid, dam, dmess);
	pr(mesg); pr("\n");
	wu(0, def_own, mesg);
    }
    pp->pln_effic = eff;
    if (disp == 1) {
	if (from != 0)
	    nreport(from, 
		    (plp->pcp->pl_flags & P_M) ? N_DOWN_MISSILE : N_DOWN_PLANE,
		    pp->pln_own, 1);
	pp->pln_own = 0;
    }
    putplane(pp->pln_uid, pp);
    if (disp > 0) {
	dpr(fmt("%s plp->kamikaze=%d",planeas(pp,LOS),plp->kamikaze));
	if ((plp->kamikaze != -1) && (((x^y) & 1)==0)) {
	    i = visiblevehicles_xy((int *)0, x, y);
	    vehlist = (int *)calloc(i+40,sizeof(int));
	    i = visiblevehicles_xy(vehlist, x, y);
	    /* randomly remove a few targets from the list */
	    for (j=k=0; j<i; j++)
		if (chance(0.33)) vehlist[k++] = vehlist[j];
	    vehlist[k] = -1;
	    if (disp==1)
		plp->kamikaze = vehlist[roll(k)];
	    if (plp->kamikaze < -1)
		list_kktargets(vehlist, x, y);
	    do_kamikaze(plp,vehlist,0);
	    free(vehlist);
	}
	remque(&plp->queue);
	free((char *)plp);
    }
}

ac_dosweep(list, sp)
struct qelem *list;
struct sctstr *sp;
{
    int mines;
    register struct plnstr *pp;
    struct plist *plp;
    struct qelem *qp,*next;
    mines = getvar(V_MINE, (char *)sp, EF_SECTOR);
    pr(fmt("sweeping %s ...\n",xyas(sp->sct_x,sp->sct_y,cnum)));
    for (qp=list->q_forw; qp!=list && mines>0; qp=next) {
	next = qp->q_forw;
	plp = (struct plist *) qp;
	pp = &plp->plane;
	if (chance(pp->pln_effic/110.0)) {
	    pr("  ");
	    switch (roll(3)) {
	    case 1: pr("kaBOOOM!"); break;
	    case 2: pr("baROOOM!"); break;
	    case 3: pr("kaPOW!"); break;
	    }
	    pr(fmt(" -- mine detonated by %s #%d\n",
		   plchr[pp->pln_type].pl_name,pp->pln_uid));
	    mines--;
	}
    }
    putvar(V_MINE, mines, (char *)sp, EF_SECTOR);
}

ac_doflak(list, from)
struct	qelem *list;
struct	sctstr *from;
{
    int	shell,gun;
    shell = getvar(V_SHELL, (char *)from, EF_SECTOR);
    gun = getvar(V_GUN, (char *)from, EF_SECTOR);
    if (gun == 0 || shell == 0)
	return;
    if (gun > shell*2)
	gun = shell*2;
    if (gun > 14)
	gun = 14;
    putvar(V_SHELL, shell - gun/2, (char *)from, EF_SECTOR);
    gun = 2.0 * tfact(from->sct_own, (double) gun);
    if (gun > 0) {
	pr(fmt("firing %d flak guns in %s...\n",
	       gun, xyas(from->sct_x, from->sct_y, cnum)));
	wu(0, from->sct_own, fmt("firing %d flak guns in %s...",
				 gun, xyas(from->sct_x, from->sct_y, from->sct_own)));
	ac_fireflak(list, from->sct_own, gun, from->sct_x, from->sct_y);
    }
}

ac_kamikaze(list, vehlist, x, y)
struct qelem *list;
coord x,y;
int *vehlist;
{
    struct shpstr ship;
    struct plist *plp;
    struct qelem *qp,*next;

    dpr(">ac_kamikaze");
    /* Print a target selection list if any plane has a -2 target */
    for (qp=list->q_forw; qp!=list; qp=next) {
	next = qp->q_forw; plp = (struct plist *) qp;
	if (plp->kamikaze < -1) {
	    list_kktargets(vehlist,x,y);
	    break;
	}
    }
    /* Process all the planes */
    for (qp=list->q_forw; qp!=list; qp=next) {
	next = qp->q_forw; plp = (struct plist *) qp;
	do_kamikaze(plp,vehlist,1);
    }
}

list_kktargets(vehlist,x,y)
int *vehlist;
coord x,y;
{
    int nvehs;
    struct shpstr ship;
    dpr("list_kktargets");
    if (vehlist && *vehlist>=0) {
	pr(fmt("Possible kamikaze targets in %s:\n",xyas(x,y,cnum)));
	for (nvehs=0; vehlist[nvehs]>=0; nvehs++) {
	    getship(vehlist[nvehs],&ship);
	    pr(fmt("  %s\n",shipas(&ship,LO)));
	}
    } else {
	dpr("Empty target list");
    }
}

do_kamikaze(plp,vehlist,dequeue)
struct plist *plp;
int *vehlist,dequeue;
{
    int i,j,dam,nvehs;
    double kkodds,kkimpact,shipsize,planesize;
    register struct plnstr *pp;
    struct shpstr ship;
    char *s;

    dpr("do_kamikaze");
    if ((plp->kamikaze == -1) || !vehlist || (*vehlist < 0)) return;
    pp = &plp->plane;
    for (nvehs=0; vehlist[nvehs]>=0; nvehs++);
    if (plp->kamikaze < 0) {
    askforkktarget:
	s = getstring(fmt("%s: kamikaze target ? ",planeas(pp,0)));
	if (s && *s) {
	    if (*s == '?') {
		list_kktargets(vehlist,pp->pln_x,pp->pln_y);
		goto askforkktarget;
	    } else if (*s == '*') {
		plp->kamikaze = vehlist[roll(nvehs)-1];
	    } else if (isdigit(*s)) {
		i = atoi(s);
		for (j=0; j<nvehs; j++) 
		    if (vehlist[j]==i) break;
		if (j<nvehs) {
		    plp->kamikaze = i;
		}
	    } else {
		pr("Specify:\n");
		pr("  <id#>  target id\n");
		pr("  <?>    show target list\n");
		pr("  <*>    pick random target\n");
		pr("  <Ret>  do nothing\n");
		goto askforkktarget;
	    }
	}
    }
    if (plp->kamikaze < 0) return;
    for (i=0; i<nvehs; i++)
	if (plp->kamikaze == vehlist[i]) break;
    if (i==nvehs) return;
    if (!getship(plp->kamikaze,&ship)) return;
    pr(fmt("%s diving at %s\n",planeas(pp,0),shipas(&ship,LOS)));

    planesize = (double)(plchr[pp->pln_type].pl_lcm + plchr[pp->pln_type].pl_hcm*2);
    if (planesize < 40.0) planesize = 40.0;

    shipsize = (double)(mchr[ship.shp_type].m_lcm + mchr[ship.shp_type].m_hcm*2);
    if (shipsize < 10.0) shipsize = 10.0;

    switch (mchr[ship.shp_type].m_class) {
    case CLASS_TANK:
	shipsize /= 3.00;
	kkimpact = (double)(400 - mchr[ship.shp_type].m_armor) / 4.0;
	break;
    case CLASS_BALLOON:
	shipsize /= 0.33;
	kkimpact = (double)(800 - mchr[ship.shp_type].m_armor) / 8.0;
	break;
    default:
	kkimpact = (double)(200 - mchr[ship.shp_type].m_armor) / 2.0;
	break;
    }

    if (kkimpact < 0.0) kkimpact = 0.0;
    kkodds = (double)pp->pln_effic * shipsize * planesize / 1000.0 / 450.0;
    dam = (int)(kkimpact * ((double)roll(50)+planesize) / shipsize);
    if (dam > 100) dam = 100;

    pr(fmt("impact odds are %.2f\n",kkodds));

    pr("  -- ");
    if (!chance(kkodds)) {
	if (isship(&ship)) pr("SPLASH!\n");
	else pr("CRASH!\n");
	wu(0, ship.shp_own, fmt("%s kamikaze exploded near %s",
				cname(cnum), shipasc(&ship,LS,ship.shp_own)));
    } else {
	nreport(cnum, N_KAMIKAZE, ship.shp_own, 1);
	pr(fmt("kaBOOOM!\n  %s takes %d\n",shipas(&ship,LO),dam));
	wu(0, ship.shp_own, fmt("%s kamikaze did %d%% damage to %s",
				cname(cnum), dam, shipasc(&ship,LS,ship.shp_own)));
	shipdamage(&ship, dam);
	putship(ship.shp_uid,&ship);
	getship(ship.shp_uid,&ship);
	if (!ship.shp_own)
	    while (i <= nvehs) vehlist[i] = vehlist[++i];
    }
    pp->pln_effic = 0;
    pp->pln_own = 0;
    putplane(pp->pln_uid, pp);
    if (dequeue) {
	remque(&plp->queue);
	free((char *)plp);
    }
}


ac_shipflak(list, x, y)
struct	qelem *list;
coord	x,y;
{
    struct	nstr_item ni;
    struct	shpstr ship;
    struct	mchrstr *mcp;
    int	firing;
    int	gun;
    int	shell;
    int	rel;
    char	*msg;
    
    snxtitem_xy(&ni, EF_SHIP, x, y);
    while (!QEMPTY(list) && nxtitem(&ni, (char *)&ship)) {
	if (ship.shp_own == 0 || ship.shp_own == cnum)
	    continue;
	mcp = &mchr[ship.shp_type];
	if (mcp->m_flags & M_SUB) /* subs never fire flak */
	    continue;
	rel = getrel(getnatp(ship.shp_own), cnum);
	if (rel != AT_WAR && rel != HOSTILE)
	    continue;
	gun = min(getvar(V_GUN, (char *)&ship, EF_SHIP), mcp->m_glim);
	shell = getvar(V_SHELL, (char *)&ship, EF_SHIP);
	if (gun == 0 || shell == 0)
	    continue;
	firing = (int) (techfact(ship.shp_tech, (double)gun) * 4.0);
	msg = fmt("firing %d flak guns from %s #%d...",
		  firing, mcp->m_name, ship.shp_uid);
	pr(msg); pr("\n");
	wu(0, ship.shp_own, msg);
	ac_fireflak(list, ship.shp_own, firing, x, y);
	putvar(V_SHELL, shell - 1, (char *)&ship, EF_SHIP);
	putship(ni.cur, &ship);
    }
}

/*
 * Called from shipflak and doflak.
 */
ac_fireflak(list, from, guns, x, y)
    struct qelem *list;
    natid from;
    int	guns;
    coord x,y;
{
	register struct plnstr *pp;
	struct	plist *plp;
	int	n;
	float	mult;
	int	diff;
	struct	qelem *qp;
	struct	qelem *next;

	for (qp = list->q_forw; qp != list; qp = next) {
		/*
		 * fighters don't get shot at by flak
		 * non-tactical bombers are harder to hit with flak.
		 * ('Cause they're not dive-bombing?)
		 */
		next = qp->q_forw;
		plp = (struct plist *) qp;
		pp = &plp->plane;
		diff = guns - pp->pln_def;

		/* don't bother with high altitude aircraft */
		if (plp->pcp->pl_flags & P_HA) continue;

		if ((plp->pcp->pl_flags & P_T) == 0)
			diff--;
		if (plp->pcp->pl_flags & P_X)
			diff -= 2;
		if (diff > 8)
			mult = flaktable[FLAK_MAX] * 1.33;
		else if (diff < -7)
			mult = flaktable[0] * 0.66;
		else {
			diff += 7;
			mult = flaktable[diff];
		}
		n = (int) ((roll(flak_range)+flak_base) * mult * flak_factor);
		if (n > 100) n = 100;
		ac_planedamage(plp, from, (int)(n * pp->pln_effic / 100.0),
			from, x, y);
	}
}

/*
 * Get a list of planes available for interception duties.
 */
static getilist(list, own)
struct	qelem *list;
natid	own;
{
    register struct plchrstr *pcp;
    struct	plnstr plane;
    struct	shpstr ship;
    struct	sctstr sect;
    struct	nstr_item ni;
    int	type;
    char	*ptr;
    struct	plist *ip;
    
    initque(list);
    snxtitem_all(&ni, EF_PLANE);
    while (nxtitem(&ni, (char *)&plane)) {
	pcp = &plchr[plane.pln_type];
	if (plane.pln_own != own)        continue;
	if (plane.pln_flags & PLN_BOXED) continue;
	if ((pcp->pl_flags & P_F) == 0)  continue;
	if (plane.pln_mobil <= 0)        continue;
	if (plane.pln_effic < 40)        continue;
	if (plane.pln_ship >= 0) {
	    if (!getship(plane.pln_ship, &ship) ||
		ship.shp_own != plane.pln_own ||
		!planefitsship(pcp->pl_flags,mchr[ship.shp_type].m_flags))
		continue;
	    ptr = (char *)&ship;
	    type = EF_SHIP;
	} else {
	    getsect(plane.pln_x, plane.pln_y, &sect);
	    ptr = (char *)&sect;
	    type = EF_SECTOR;
	    if ((pcp->pl_flags & P_V) == 0) {
#ifdef TRADECENTERISAIRPORT
		if (sect.sct_type!=SCT_AIRPT &&
		    sect.sct_type!=SCT_TRADE) continue;
#else		
		if (sect.sct_type != SCT_AIRPT) continue;
#endif
		if (sect.sct_effic < 60) continue;
	    }
	}
	if (getvar(V_PETROL, ptr, type) < pcp->pl_fuel/2)
	    continue;
	/* got one! */
	ip = (struct plist *) malloc(sizeof(*ip));
	ip->state = P_OK;
	ip->bombs = 0;
	ip->misc = 0;
	ip->pcp = &plchr[plane.pln_type];
	bcopy((char *)&plane, (char *)&ip->plane, sizeof(plane));
	insque(&ip->queue, list);
    }
}
