#ifndef lint
static char *RCSid = "$Header: comsub.c,v 1.1 89/12/14 13:28:12 jay Exp $";
#endif

/*
 * comsub.c
 *
 * commodity read/write stuff.
 *
 * Dave Pare, 1986
 * Trade center additions by Bill Frolik, 1990
 */

#include "misc.h"
#include "var.h"
#include "nat.h"
#include <stdio.h>
#include <fcntl.h>
#include <sys/uio.h>

#ifdef TRADECENTERS

/* ________________________________________________________________
**|                                                                |
**|             T R A D E   C E N T E R   M A R K E T              |
**|________________________________________________________________|
*/

#include "shop.h"

int commf;
static off_t shoppos;

int lockshop() { 
    int n;
    dpr(fmt("lockshop() returning %d",n=file_lock(commf)));
    return n;
}

int unlockshop() {
    int n;
    dpr(fmt("unlockshop() returning %d",n=file_unlock(commf)));
    return n;
}

int shopread(shop,num)
/* Read trade center structure number 'num' */
/* If 'num' < 0, read structure at current 'shoppos' offset */
/* Return TRUE if read was successful */
tradecenter *shop;
int num;
{
    int n;
    if (num >= 0) shoppos = (off_t)(num * SOT);
    dpr(fmt("shopread(0x%lx,%d)",shop,num)); dpr(">");
    dpr(fmt("shoppos = 0x%ld, limit = 0x%ld",shoppos,SOT*MAXTRADECENTERS));
    n = 0;
    if (shoppos >= (SOT*(MAXTRADECENTERS)))
	logerror("shopread: attempted read beyond %d entries\n",
		 MAXTRADECENTERS);
    else {
	if (lseek(commf,shoppos,0) == -1)
	    logerror("shopread: seek failed\n");
	else {
	    n = read(commf,shop,SOT);
	    if (n==SOT) dpr(fmt("read shop @ %d,%d",shop->x,shop->y));
	    else dpr(fmt("read failed (returned %d, expected %d)",n,SOT));
	}
    }
    dpr("<");
    return(n==SOT);
}

int shopwrite(shop,num)
/* Write a trade center structure at file offset 'shoppos' */
tradecenter *shop;
int num;
{
    int n;
    if (num >= 0) shoppos = (off_t)(num * SOT);
    dpr(fmt("shopwrite(0x%lx,%d)",shop,num)); dpr(">");
    dpr(fmt("shoppos = 0x%ld, limit = 0x%ld",shoppos,SOT*MAXTRADECENTERS));
    n = 0;
    if (shoppos >= (SOT*(MAXTRADECENTERS)))
	logerror("shopwrite: attempted write beyond %d entries\n",
		 MAXTRADECENTERS);
    else {
	if (lseek(commf,shoppos,0) == -1)
	    logerror("shopwrite: seek failed\n");
	else {
	    n = write(commf,shop,SOT);
	    if (n==SOT) dpr(fmt("wrote shop @ %d,%d",shop->x,shop->y));
	    else dpr(fmt("write failed (returned %d, expected %d)",n,SOT));
	}
    }
    dpr("<");
    return(n==SOT);
}

int shopstatus(shop,cnum,isopen)
/* Open or close a trade center to a specific country */
int cnum,isopen;
tradecenter *shop;
{
    UC mask;
    int index,n;
    if (cnum>MAXNOC) return;
    mask = bit(cnum % SUC);
    index = cnum/SUC;
    dpr(fmt(">shopstatus(0x%lx,%d,%d) mask=0x%x, index=%d",
	    shop,cnum,isopen,mask,index));
    switch (isopen) {
    case 0: shop->openflags[index] = shop->openflags[index] & ~mask; break;
    case 1: shop->openflags[index] = shop->openflags[index] | mask; break;
    default: dpr(fmt("no switch case %d",isopen));
    }
    n = (shop->openflags[index] & mask) ? 1 : 0;
    dpr(fmt("returning %s",n?"OPEN":"CLOSED")); 
    dpr("<");
    return (n);
}

int goodshop(shop)
tradecenter *shop;
{
    int n;
    n = (((shop->x ^ shop->y) & 1) == 0);
    dpr(fmt("goodshop(0x%lx) shop @ %d,%d returning %d",
	    shop,shop->x,shop->y,n));
    return(n);
}

int getshop(x,y,shop)
/* Try to get trade center at x,y & return TRUE if successful */
coord x,y;
tradecenter *shop;
{
    int n;
    tradecenter tempshop;
    dpr(fmt("getshop(%d,%d,0x%lx)",x,y,shop)); dpr(">");
    if (!shop) shop = &tempshop;
    shoppos = (off_t) 0;
    n = lseek(commf, shoppos, 0);
    dpr(fmt("lseek 0 returned %d",n));
    while ((n=read(commf,shop,SOT)) == SOT) {
	dpr(fmt("(pos=%ld) read ok shop @ %d,%d",shoppos,shop->x,shop->y));
	if (shop->x==x && shop->y==y) {
	    lseek(commf, shoppos, 0); /* got it, back up */
	    dpr(fmt("good shop, lseeking back to %ld",shoppos));
	    break;
	}
	else shoppos += SOT;
	dpr(fmt("pos is now %ld",shoppos));
    }
    dpr(fmt("returning %s",(n==SOT)?"TRUE":"FALSE"));
    dpr("<");
    return (n==SOT);
}

int putshop(shop)
/* Put a tradecenter into the database */
tradecenter *shop;
{
    int n;
    tradecenter tempshop;
    dpr(fmt("putshop(0x%lx)",shop)); dpr(">");
    if (!getshop(shop->x,shop->y,&tempshop)) {
	/*  Scan for a reusable record (or past EOF) */
	shoppos = (off_t) 0;
	lseek(commf, shoppos, 0);
	while ((n=read(commf,&tempshop,SOT)) == SOT) {
	    if (!goodshop(&tempshop)) {
		lseek(commf,shoppos,0); /* got one, back up */
		break;
	    }
	    else shoppos += SOT;
	    dpr(fmt("pos is now %ld",shoppos));
	}
    }
    n = shopwrite(shop,-1);
    dpr("<");
    return(n);
}

int setupshop(x,y,shop)
/* Initialize a trade center structure and put it in the database */
coord x,y;
tradecenter *shop;
{
    int i,j;
    dpr(fmt("setupshop(%d,%d,0x%lx)",x,y,shop));
    shop->x = x;
    shop->y = y;
    for (j=0;j<MAXNOC;j++) {
	shopstatus(shop,j,0);
	shop->pricemult[j] = 1.0;
    }
    shopstatus(shop,cnum,1);            /* open to owner */
    for (i=0;i<=I_MAX;i++) {
	shop->item[i].buyprice = -1.0;	/* -1 = not buying   */
	shop->item[i].sellprice = -1.0; /* -1 = not for sale */
	shop->item[i].lowater = 0;      /* min to keep in sector */
	shop->item[i].hiwater = 999;    /* max we want in sector */
    }
    
    if (!putshop(shop)) { /* shoppos is also now set */
	pr("Can't initialize trade center; get help!\n");
	logerror(fmt("setupshop(%d,%d) failed",x,y));
	return 0;
    }
    return 1;
}

int destroyshop(x,y)
/* Remove from the database the trade center at x,y */
coord x,y;
{
    tradecenter t;
    dpr(fmt("destroyshop(%d,%d)",x,y));
    t.x = 0; t.y = 1;
    if (getshop(x,y,0)) return(shopwrite(&t,-1));
    else return(0);
}

int createshop(x,y)
/* Add to the database a new trade center at x,y */
coord x,y;
{
    int n;
    tradecenter shop;
    lockshop();
    if (getshop(x,y,&shop)) n = 1;
    else n = setupshop(x,y,&shop);
    unlockshop();
    return n;
}

int shopisopen(x,y,towhom)
coord x,y;
natid towhom;
{
    int i;
    off_t sp;
    tradecenter shop;
    sp = shoppos; lockshop();
    i = getshop(x,y,&shop);
    shoppos = sp; unlockshop();
    if (!i) return(0);
    return(shopstatus(&shop,towhom,2));
}

#else

/* ________________________________________________________________
**|                                                                |
**|                 T E L E P O R T   M A R K E T                  |
**|________________________________________________________________|
*/

int   commf;

int   c_comm[MAXNOC][I_MAX+1];
float c_price[MAXNOC][I_MAX+1];

static	struct iovec commvec[2] = {
    (caddr_t)c_comm,	sizeof(c_comm),
    (caddr_t)c_price,	sizeof(c_price),
};

static	int nvec = sizeof(commvec)/sizeof(struct iovec);

/*
** comm file is the list of "deity commodities"  lying around. 
** Perhaps a better way of doing this might be at each update  
** seeing what is on sale, then buying it, then distributing   
** things amongst the people wanting the items.  Of course,    
** that would be a gigantic pain...                            
*/

int commlock() { return file_lock(commf); }

int communlock() { return file_unlock(commf); }

int commread()
{
    int	n;
    (void) lseek(commf, (off_t) 0, 0);
    if ((n = readv(commf, commvec, nvec)) !=
	sizeof(c_comm) + sizeof(c_price)) {
	logerror("commread: readv returns %d", n);
	return -1;
    }
    return 0;
}

int commwrite()
{
    (void) lseek(commf, (off_t) 0, 0);
    if (writev(commf, commvec, nvec) != 
	sizeof(c_comm) + sizeof(c_price)) {
	logerror("commwrite: writev failed");
		return -1;
    }
    return 0;
}

/* Return amount of commodity, and price to user */
int commamt(trader, product, priceval)
natid trader;
int product;
float *priceval;
{
    *priceval = c_price[trader][product] * (double)multread(trader,cnum);
    return c_comm[trader][product];
}

commset(trader, product, amt)
natid trader;
int product,amt;
{ c_comm[trader][product] += amt; }

commprice(product, newprice)
int product;
float *newprice;
{ c_price[cnum][product] = *newprice; }

#endif
