#ifndef lint
static char *RCSid = "$Header: move.c,v 1.1 89/12/14 13:28:21 jay Exp $";
#endif

/*
 * move.c
 *
 * Move something somewhere.  Has a weight,
 * returns move cost, and fills a result parameter
 * with the destination sector.
 *
 * 4/15/91 bf@hp - added exploration cost/sector and minimum mobility usage.
 *                 Note that the 'exploring' parameter is no longer a simple
 *                 int, but is instead now a pointer to the double that 
 *		   tells the minimum mobility to be charged per sector.
 *
 */

#include "misc.h"
#include "var.h"
#include "sect.h"
#include "item.h"
#include "file.h"
#include "deity.h"
#include "xy.h"
#include "path.h"
#include "nat.h"

extern double explorecost; /* exploration cost per sector */

int
move_ground(what, start, end, mobility, weight, path, map, exploring)
char *what;
struct sctstr *start,*end;
double mobility,weight;
char *path;
int (*map)();
double *exploring;
{
    struct sctstr sect,next;
    struct natstr *natp;
    int	vec[I_MAX+1];
    coord curx, cury;
    coord tmpx, tmpy;
    char *movstr;
    double sect_mcost,total_mcost,total_dcost,cost_notify;
    int	mcost,dir,intcost;

    movstr = path;
    tmpx = start->sct_x;
    curx = tmpx;
    tmpy = start->sct_y;
    cury = tmpy;
    total_mcost = total_dcost = 0.0;
    natp = getnatp(cnum);
    cost_notify = 0.05;

    if (getsect(curx, cury, &sect) < 0) {
	logerror("move_path: getsect %d,%d", curx, cury);
	return -1;
    }
    for (;;) {
	tmpx = curx;
	tmpy = cury;

	if (exploring) {
	    if (total_dcost > (double)natp->nat_money) {
		pr(fmt("Exploration funds exhausted at %s\n",
		       xyas(sect.sct_x, sect.sct_y, cnum)));
		movstr = dirch;
	    } else {
		if (total_dcost > (double)natp->nat_money*cost_notify) {
		    pr(fmt("Warning: Exploration cost now $%.2f",total_dcost));
		    pr(fmt(" (exceeds %2d%% of available funds)\n",
			   (int)(cost_notify*100.0)));
		    cost_notify += 0.05;
		}
	    }
	}

	if (movstr == 0 || *movstr == 0) {
	    if (exploring) map(what, curx, cury, (char *)0);
	    movstr = getstring(fmt("<%.1f: %c %s> ", mobility,
				   dchr[sect.sct_type].d_mnem,
				   xyas(sect.sct_x, sect.sct_y, cnum)));
	}

	if (movstr == 0 || *movstr == 0)
	    movstr = dirch;

	if (exploring && (*movstr == '$')) {
	    pr(fmt("Expedition cost so far: $%.2f\n",
		   total_dcost));
	    pr(fmt("Total funds remaining:  $%.2f\n\n",
		   (double)natp->nat_money-total_dcost));
	    movstr++;
	    continue;
	}

	if ((dir = chkdir(*movstr, DIR_STOP, DIR_MAP)) < 0) {
	    pr(fmt("\"%c\" is not legal...", *movstr));
	    direrr("'%c' to stop ", "'%c' to view ",
		   (exploring) ? "'%c' to map '$' to see cost\n\n"
		   : "& '%c' to map\n\n");
	    *movstr = 0;
	    continue;
	}

	movstr++;
	if (dir == DIR_MAP) {
	    if (!exploring)
		map(what, curx, cury, movstr+1);
	    *movstr = 0;
	    continue;
	} else if (dir == DIR_STOP)
	    break;
	else if (dir == DIR_VIEW) {
	    pr(fmt("%d%% %s with %d civilians, %d military and %d food.\n",
		   sect.sct_effic,
		   dchr[sect.sct_type].d_name,
		   getvar(V_CIVIL, (char *)&sect, EF_SECTOR),
		   getvar(V_MILIT, (char *)&sect, EF_SECTOR),
		   getvar(V_FOOD, (char *)&sect, EF_SECTOR)));
	    continue;
	}
	/*
	 * now see if we can move into the
	 * next sector.  Mobility, terrain,
	 * or ownership may prevent us.
	 */
	tmpx += diroff[dir][0];
	tmpy += diroff[dir][1];
	if (getsect(tmpx, tmpy, &next) < 0) {
	    pr("You can't go there...\n");
	    *movstr = 0;
	    continue;
	}
	if (!god) {
	    if ((next.sct_type == SCT_SANCT) &&
		(next.sct_own != cnum)) {
		pr("Converts, huh?\n");
		*end = next;
		return total_mcost;
	    }
	    getvec(VT_ITEM, vec, (char *)&next, EF_SECTOR);
	    mcost = dchr[next.sct_type].d_mcst;
	    if ((!owner && (!exploring || 
			    (vec[I_MILIT] || vec[I_CIVIL]))) ||
		mcost == 0) {
		/* already-owned, or prohibited terrain */
		pr("You can't go there...\n");
		*movstr = 0;
		continue;
	    }
	    sect_mcost = (mcost * 100 - next.sct_effic) / 500.0;
	    sect_mcost *= weight;
/*	    if (sect_mcost < *exploring) sect_mcost = *exploring; */
	    if (sect_mcost > mobility) {
		pr("Not enough mobility.  ");
		pr("You can't go there...\n");
		*movstr = 0;
		continue;
	    }
	    mobility -= sect_mcost;
	    total_mcost += sect_mcost;
	    if (exploring && (next.sct_own != cnum)) 
		total_dcost += explorecost;
	}
	curx = tmpx;
	cury = tmpy;
	sect = next;
    }
    *end = sect;
    intcost = (int) total_mcost;
    if (intcost < 0)
	return -1;
    if (chance(total_mcost - intcost))
	intcost++;
    if (explorecost>0.0 && total_dcost>0.0)
	dolcost += total_dcost+0.5;
    return intcost;
}
