#ifndef lint
static char *RCSid = "$Header: show.c,v 1.1 89/12/14 13:28:38 jay Exp $";
#endif

/*
 * show.c
 *
 * general routines that are callable to give info on things.
 * currently, planes, ships, nukes and bridges. Tanks & regiments one day?
 *
 * Julian Onions, 1988
 */

#include "misc.h"
#include "nuke.h"
#include "var.h"
#include "ship.h"
#include "item.h"
#include "plane.h"
#include "spy.h"
#include "deity.h"

extern double nuke_cost_multiplier;
extern double nuke_tech_multiplier;
extern double plane_cost_multiplier;
extern double plane_tech_multiplier;
extern double ship_cost_multiplier;
extern double ship_tech_multiplier;
extern double spy_cost_multiplier;
extern double spy_tech_multiplier;
extern int etu_per_update;
extern int bal_refill_rate;
extern int sub_refill_rate;

/*
 * This cruft really belongs in the empglb.c file.
 * Yuck.
 */

struct lookup {
    int     key;
    char   *value;
};

struct lookup  ship_flags[] = {
    M_FOOD, "fish",	    M_TORP, "torp",	    M_DCH, "dchrg",
    M_FLY, "plane",	    M_MSL, "miss",	    M_OIL, "oil",
    M_SONAR, "sonar",	    M_MINE, "mine",	    M_SWEEP, "sweep",
    M_SUB, "sub",	    M_SPY, "spy",	    M_LAND, "land",
    M_ATV, "atv",	    M_VTOL, "vtol",	    M_HELI, "heli",
    M_TAC, "tran",
    0, 0
};

struct lookup  plane_flags[] = {
    P_E, "enhanced",	    P_T, "tactical",	    P_B, "bomber",
    P_F, "intercept",	    P_C, "cargo",	    P_V, "VTOL",
    P_M, "missile",	    P_L, "light",	    P_S, "spy",
    P_I, "image",	    P_O, "satellite",	    P_X, "stealth",
    P_U, "subhunt",	    P_H, "heli",	    P_MS,"sweep",
    0, 0
    };

struct lookup spy_flags[] = {
    S_A, "assassin",	    S_B, "bomber",	    S_I, "informant",
    0, 0 
};

static char *lookup(key, table)
int key;
struct lookup *table;
{
    int	match;
    
    if ((match=intmatch(key,&table->key,sizeof(*table))) < 0) return 0;
    return table[match].value;
}

int *shipsbytech()
{
    int i,j,n,*vi;
    vi  = (int *)malloc(shp_maxno*sizeof(int));
    for (n=0; n<shp_maxno; n++) vi[n] = n;
    for (i=0; i<shp_maxno-1; i++)
	for (j=i+1; j<shp_maxno; j++)
	    if (mchr[vi[i]].m_tech > mchr[vi[j]].m_tech) {
		n=vi[i]; vi[i]=vi[j]; vi[j]=n;
	    }
    return(vi);
}

int *spiesbytech()
{
    int i,j,n,*vi;
    vi  = (int *)malloc(spy_maxno*sizeof(int));
    for (n=0; n<spy_maxno; n++) vi[n] = n;
    for (i=0; i<spy_maxno-1; i++)
	for (j=i+1; j<spy_maxno; j++)
	    if (spchr[vi[i]].sp_tech > spchr[vi[j]].sp_tech) {
		n=vi[i]; vi[i]=vi[j]; vi[j]=n;
	    }
    return(vi);
}

int *planesbytech()
{
    int i,j,n,*vi;
    vi  = (int *)malloc(pln_maxno*sizeof(int));
    for (n=0; n<pln_maxno; n++) vi[n] = n;
    for (i=0; i<pln_maxno-1; i++)
	for (j=i+1; j<pln_maxno; j++)
	    if (plchr[vi[i]].pl_tech > plchr[vi[j]].pl_tech) {
		n=vi[i]; vi[i]=vi[j]; vi[j]=n;
	    }
    return(vi);
}

int *nukesbytech()
{
    int i,j,n,*vi;
    vi  = (int *)malloc(N_MAXNUKE*sizeof(int));
    for (n=0; n<N_MAXNUKE; n++) vi[n] = n;
    for (i=0; i<N_MAXNUKE-1; i++)
	for (j=i+1; j<N_MAXNUKE; j++)
	    if (nchr[vi[i]].n_tech > nchr[vi[j]].n_tech) {
		n=vi[i]; vi[i]=vi[j]; vi[j]=n;
	    }
    return(vi);
}

show_bridge(tlev)
int tlev;
{
    extern double buil_bt, buil_bc;
    extern int buil_bh;
    
    if (tlev < buil_bt)	return;
    pr(fmt("Bridges require %g tech, %d hcm, %d workers,\n",
	   buil_bt, buil_bh, buil_bh * 2));
    pr(fmt("%d available workforce, and cost $%g\n",
	   1 + (buil_bh * 40 / 100), buil_bc));
}

show_nuke_build(tlev)
int tlev;
{
    register struct nchrstr *np;
    register int n;
    register int avail;
    int *vi;
    vi = nukesbytech();
    pr(fmt("%13s lcm hcm  oil  rad avail tech $\n", ""));
    for (n=0; n<N_MAXNUKE; n++) {
	np = &nchr[vi[n]];
	if (np->n_tech > tlev && !god) continue;
	if (np->n_name == 0 || np->n_name[0] == '\0') continue;
	avail = 1 +
	    (np->n_rad + np->n_oil + np->n_lcm + np->n_hcm * 2)*20/100;
	pr(fmt("%-13.13s %3d %3d %4d %4d %5d %4d $%6d\n",
	       np->n_name, np->n_lcm, np->n_hcm, np->n_oil, 
	       np->n_rad, avail,
	       (int)(((double)np->n_tech)*nuke_tech_multiplier), 
	       (int)(((double)np->n_cost)*nuke_cost_multiplier)
	       ));
    }
    free(vi);
}

show_nuke_capab(tlev)
int tlev;
{
    register struct nchrstr *np;
    register int n;
    int *vi;
    vi = nukesbytech();
    pr(fmt("%13s blst dam lbs tech $\n", ""));
    for (n=0; n<N_MAXNUKE; n++) {
	np = &nchr[vi[n]];
	if (np->n_tech > tlev && !god) continue;
	if (np->n_name == 0 || np->n_name[0] == '\0') continue;
	pr(fmt("%-13.13s %4d %3d %3d%5d $%d\n",
	       np->n_name, np->n_blast, np->n_dam,
	       np->n_weight, 
	       (int)(((double)np->n_tech)*nuke_tech_multiplier), 
	       (int)(((double)np->n_cost)*nuke_cost_multiplier)
	       ));
    }
    free(vi);
}

show_sub_volume(tlev)
int tlev;
{
    register struct mchrstr *mp;
    register int n;
    int vec[I_MAX+1];
    int *vi;
    vi = shipsbytech();
    pr(fmt("%13s        depletion rate     max     refill\n",""));
    pr(fmt("%13s tech    (per person)    air vol    rate\n",""));
    for (n=0; n<shp_maxno; n++) {
	mp = &mchr[vi[n]];
	if (!(mp->m_flags & M_SUB)) continue;
	if (mp->m_tech > tlev && !god) continue;
	if (!mp->m_name || !*(mp->m_name)) continue;
	pr(fmt("%-13.13s %4d %12d %12d %8d\n",
	       mp->m_name,
	       (int)((double)mp->m_tech*ship_tech_multiplier),
	       etu_per_update,
	       mp->m_maxair,
	       sub_refill_rate * etu_per_update
	       ));
    }
    free(vi);
}

show_balloon_volume(tlev)
int tlev;
{
    register struct mchrstr *mp;
    register int n;
    int *vi;
    vi = shipsbytech();
    pr(fmt("%13s            min gas req'd       max     refill\n",""));
    pr(fmt("%13s tech     low alt  high alt   gas vol    rate\n",""));
    for (n=0; n<shp_maxno; n++) {
	mp = &mchr[vi[n]];
	if (mp->m_class != CLASS_BALLOON) continue;
	if (mp->m_tech > tlev && !god) continue;
	if (!mp->m_name || !*(mp->m_name)) continue;
	pr(fmt("%-13.13s %4d %9d %9d %9d %9d\n",
	       mp->m_name,
	       (int)((double)mp->m_tech*ship_tech_multiplier),
	       mp->m_maxair / 2,
	       mp->m_maxair * 3 / 4,
	       mp->m_maxair,
	       bal_refill_rate * etu_per_update
	       ));
    }
    free(vi);
}

show_vehicle_build(tlev,mflag)
int tlev,mflag;
{
    extern double money_ship;
    extern int etu_per_update;
    register struct mchrstr *mp;
    register int n;
    double cost;
    int *vi;
    vi = shipsbytech();
    pr(fmt("%13s lcm hcm avail tech  $     maint\n", ""));
    for (n=0; n<shp_maxno; n++) {
	mp = &mchr[vi[n]];
	if (mflag != mp->m_class) continue;
	if (mp->m_tech > tlev && !god) continue;
	if (mp->m_name == 0 || mp->m_name[0] == '\0') continue;
	/* Calc per update maintenance cost. */
	cost = dmin(-1.0, mp->m_cost * money_ship);
	cost = cost * (-etu_per_update);
	pr(fmt("%-13.13s %3d %3d %5d %4d $%-6d $%d\n",
	       mp->m_name, mp->m_lcm, mp->m_hcm,
	       (int) (1 + (2 * mp->m_hcm + mp->m_lcm) * 20 / 100),
	       (int)((double)mp->m_tech*ship_tech_multiplier),
	       (int)((double)mp->m_cost*ship_cost_multiplier),
	       (int)cost));
    }
    free(vi);
}

show_vehicle_capab(tlev,mflag)
int tlev,mflag;
{
    register struct mchrstr *mp;
    register u_short *ap;
    register u_char *type;
    register int i;
    register int it;
    int scount,n;
    char c,*p;
    int *vi;

    vi = shipsbytech();
    pr(fmt("%13s def spd vis spy rng fir mpg tnk pln cargoes & abilities\n", ""));
    for (scount=0; scount<shp_maxno; scount++) {
	mp = &mchr[vi[scount]];
	if (mflag != mp->m_class) continue;
	if (mp->m_tech > tlev && !god) continue;
	if (mp->m_name == 0 || mp->m_name[0] == '\0') continue;
	pr(fmt("%-13.13s %3d %3d %3d %3d %3d %3d %3d %3d %3d",
	       mp->m_name, mp->m_armor, mp->m_speed,
	       mp->m_visib, mp->m_vrnge, mp->m_frnge,
	       mp->m_glim, mp->m_mpg, mp->m_maxtnks,
	       mp->m_maxplns));
	/*
	 * should use vector stuff
	 */
	for (ap = mp->m_vamt, type = mp->m_vtype, i = 0;
	     i < mp->m_nv; i++, ap++, type++) {
	    it = unitem((int) *type);
	    if (it > 0 && it <= I_MAX && ichr[it].i_name != 0)
		c = ichr[it].i_name[0];
	    else
		c = '?';
	    pr(fmt(" %d%c", *ap, c));
	}
	pr(" ");
	for (i = n = 0; i < 32; i++) {
	    if (!(mp->m_flags & bit(i))) continue;
	    if (p = lookup(bit(i), ship_flags)) {
		if (n++) pr(" ");
		pr(p);
	    }
	}
	pr("\n");
    }
    free(vi);
}

show_plane_capab(tlev)
int tlev;
{
    register struct plchrstr *pp;
    register int i;
    int pcount,n;
    char *p;
    int *vi;
    vi = planesbytech();
    pr(fmt("%15s acc load att def ran fuel abilities\n", ""));
    for (pcount=0; pcount<pln_maxno; pcount++) {
	pp = &plchr[vi[pcount]];
	if (pp->pl_tech > tlev && !god) continue;
	if (pp->pl_name == 0 || pp->pl_name[0] == '\0') continue;
	pr(fmt("%-13.13s %3d %4d %3d %3d %3d %4d ",
	       pp->pl_name, pp->pl_acc, pp->pl_load,
	       pp->pl_att, pp->pl_def, pp->pl_range,
	       pp->pl_fuel));
	for (i = n = 0; i < 32; i++) {
	    if (!(pp->pl_flags & bit(i))) continue;
	    if (p = lookup(bit(i), plane_flags)) {
		if (n++) pr(" ");
		pr(p);
	    }
	}
	pr("\n");
    }
    free(vi);
}

show_plane_build(tlev)
int tlev;
{
    extern double money_mil;
    extern double money_plane;
    extern int etu_per_update;
    register struct plchrstr *pp;
    register int n;
    double cost;
    int *vi;
    vi = planesbytech();
    pr(fmt("%15s lcm hcm crew avail tech  $     maint\n",""));
    for (n=0; n<pln_maxno; n++) {
	pp = &plchr[vi[n]];
	if (pp->pl_tech > tlev && !god) continue;
	if (pp->pl_name == 0 || pp->pl_name[0] == '\0') continue;
	/* Calc per update maintenance cost. */
	cost = (5 * pp->pl_crew * money_mil) +
	    dmin(-1.0, pp->pl_cost * money_plane);
	cost = cost * (-etu_per_update);
	pr(fmt("%-13.13s %3d %3d %3d %5d %4d $%-6d $%d\n",
	       pp->pl_name, pp->pl_lcm,
	       pp->pl_hcm, pp->pl_crew,
	       (int) (1 + (2 * pp->pl_hcm + pp->pl_lcm) * 50 / 100),
	       (int)((double)pp->pl_tech*plane_tech_multiplier),
	       (int)((double)pp->pl_cost*plane_cost_multiplier),
	       (int)cost));
    }
    free(vi);
}

show_spy_capab(tlev)
int tlev;
{
    register struct spchrstr *scp;
    register int i;
    int pcount,n;
    char *p;
    int *vi;
    vi = spiesbytech();
    pr(fmt("%13s  acc  abilites\n", ""));
    for (pcount=0; pcount<spy_maxno; pcount++) {
	scp = &spchr[vi[pcount]];
	if (scp->sp_tech>tlev && !god) continue;
	if (!scp->sp_name || !*scp->sp_name) continue;
	pr(fmt("%-13.13s %3d ", scp->sp_name, scp->sp_acc));
	for (i = n = 0; i < 32; i++) {
	    if (!(scp->sp_flags & bit(i))) continue;
	    if (p = lookup(bit(i), spy_flags)) {
		if (n++) pr(" ");
		pr(p);
	    }
	}
	pr("\n");
    }
    free(vi);
}

show_spy_build(tlev)
int tlev;
{
    extern double money_spy;
    extern int etu_per_update;
    register struct spchrstr *scp;
    register int n;
    double cost;
    int *vi;
    vi = spiesbytech();
    pr(fmt("%13s lcm hcm bar tech  $\n",""));
    for (n=0; n<spy_maxno; n++) {
	scp = &spchr[vi[n]];
	if (scp->sp_tech > tlev && !god) continue;
	if (!scp->sp_name || !*(scp->sp_name)) continue;
	pr(fmt("%-13.13s %3d %3d %3d %4d $%-6d\n",
	       scp->sp_name, scp->sp_lcm, scp->sp_hcm, scp->sp_bar,
	       (int)((double)scp->sp_tech*spy_tech_multiplier),
	       (int)((double)scp->sp_cost*spy_cost_multiplier)));
    }
    free(vi);
}

show_blast_profile(np)
struct nchrstr *np;
{
    int i,n,m;
    int gdam,adam,gdone,adone;
    char buf1[80],buf2[80];

    sprintf(buf1,"%-18.18s%s Bomb Blast Profile","",np->n_name);
    pr(fmt("\n\n%s\n\n",buf1));
    
    pr("     GROUND   % DAMAGE          1");
    pr("     AIR      % DAMAGE          1\n");
    strcpy(buf1,"     BURST    1 2 3 4 5 6 7 8 9 0");
    pr(fmt("%s%s\n",buf1,buf1));
    strcpy(buf1,"     dam rad _0_0_0_0_0_0_0_0_0_0");
    pr(fmt("%s%s\n",buf1,buf1));
    gdone = adone = 0;
    for (i=0;!(gdone && adone);i++) {
	gdam = nukedamage(np,i,0);
	adam = nukedamage(np,i,1);
	if (gdam==0) gdone++;
	if (adam==0) adone++;
	sprintf(buf1,"%-33.33s","");
	if (!gdone) {
	    sprintf(buf1,"%8d%3d |",gdam,i);
	    if (gdam>100) gdam=100;
	    for (n=5;n<=100;n+=5)
		strcat(buf1,(n<=gdam) ? "@" : " ");
	}
	sprintf(buf2,"%-33.33s","");
	if (!adone) {
	    sprintf(buf2,"%8d%3d |",adam,i);
	    if (adam>100) adam=100;
	    for (n=5;n<=100;n+=5)
		strcat(buf2,(n<=adam) ? "@" : " ");
	}
	pr(fmt("%s%s\n",buf1,buf2));
    }
    pr("\n");
}
