#ifndef lint
static char *RCSid = "$Header: nat.c,v 1.1 89/12/14 13:30:47 jay Exp $";
#endif

/*
 * nat.c
 *
 * Accumulate tech, edu, research, and happiness.
 * Share them among allies.
 * 
 * and Dave Pare, 1989
 */

#include <math.h>
#include "misc.h"
#include "var.h"
#include "sect.h"
#include "nat.h"
#include "item.h"
#include "news.h"
#include "file.h"
#include "xy.h"
#ifdef LOTTERY
#include "lotto.h"
#endif

float	levels[MAXNOC][4];

/*
 * hap and edu avg mean that the weight on current happiness is
 *  (cur_hap * hap_avg + hap_prod * etu) / (hap_avg + etu);
 * same for education.
 * right now, happiness has 1 day (48 etu) average, prod of 10 from
 * initial level of 0 yields (1) 1.42, (6) 6.03, (12) 8.42, (18) 9.37
 *
 * education has 4 day (192 etu) average, prod of 10 from initial
 * level of 0 yields (1) 0.4, (6) 2.2, (12) 3.9, (18) 5.2.
 */

extern float	hap_avg;
extern float	edu_avg;
extern float	ally_factor;
extern float	hard_tech;
extern float	easy_tech;
extern float	tech_log_base;

/*
 * technique to limit the sharpers who turn entire countries
 * into tech plants overnight...
 */

double limit_tech(tlev) 
double	tlev;
{
    if (tlev > easy_tech) {
	tlev -= easy_tech;
	if (tech_log_base > 1.0)  {
	    if (tlev > tech_log_base) {
		tlev = tech_log_base - 1 + log10(tlev) /
		    log10(tech_log_base);
	    }
	} else {
	    if (hard_tech > 0.0) {
		tlev = hard_tech * (tlev / (tlev +hard_tech));
	    }
	}
	tlev += easy_tech;
    }
    return tlev;
}

double limit_edu(elev, etu)
double	elev;
int	etu;
{
    /* XXX - This really needs to be tuned -- jeffw */
    if (elev > (5 * etu)) {
	elev = 5 * etu;
    }
    return elev;
}



prod_nat(etu)
int	etu;
{
    extern	long money[MAXNOC];
    extern	double money_res;
    extern	long pops[MAXNOC];
    extern	double hap_cons, edu_cons;
    extern	long sea_money[MAXNOC];
    extern	long air_money[MAXNOC];
    extern	long spy_money[MAXNOC];
    struct	natstr *np;
    float	hap;
    float	edu;
    float	hap_edu;
    float	newhap, newedu;
    long	pop;
    double	rlev;
    double	tlev;
    double	tech[MAXNOC];
    double	res[MAXNOC];
    double	newvalue;
    natid	n;

#ifdef LOTTERY
    long minmoney;
    int winners,poorman,drew;
    int i,j,jackpot,secpot;
    int picks[NUMPICKS];
    int windfall[MAXNOC];
    char lottobuf[100],buf2[15];
    if (drew=locklotto(1,20)) {
	drawlotto(&jackpot,&secpot,picks,windfall);
	secpot = (secpot>>2)*3;
	sprintf(lottobuf,"POGOLOTTO draw for $%d jackpot: ",jackpot);
	for (i=0;i<NUMPICKS;i++) {
	    sprintf(buf2,"%2d ",picks[i]);
	    strcat(lottobuf,buf2);
	}
	for (winners=i=0; i<MAXNOC; i++) if (windfall[i]) winners++;
	if (!winners) strcpy(buf2,"No winners");
	else if (winners==1) strcpy(buf2,"1 winner");
	else sprintf(buf2,"%d winners",winners);
	clearlotto(winners);
	if (winners) resetlotto(); /* insure it gets reset */
	unlocklotto();
	strcat(lottobuf," -- ");
	strcat(lottobuf,buf2);
	/* Figure out who's entitled to economic aid */
	minmoney = 123456789;
	for (poorman=n=0; np = getnatp(n); n++) {
	    if ((np->nat_stat & STAT_NORM) == 0)
		continue;
	    if (np->nat_money < minmoney) {
		minmoney = np->nat_money;
		poorman = n;
	    }
	}
    }
#endif
	
    for (n=0; np = getnatp(n); n++) {
	if ((np->nat_stat & STAT_NORM) == 0)
	    continue;
	/*
	**  hap_edu: the more education people have, the
	**  more happiness they want.
	*/
	hap_edu = np->nat_level[NAT_ELEV];
	hap_edu = 1.5 - ((hap_edu + 10.0) / (hap_edu + 20.0));
	pop = pops[n] + 1;

#ifdef LOTTERY
	/*
	** collect lottery winnings
	*/
	if (drew) {
	    wu(0,n,fmt("%s\n",lottobuf));
	    if (n==poorman && secpot>0) {
		wu(0,n,
		   fmt("POGOLOTTO economic aid fund pays you $%d.\n",secpot));
		np->nat_money += secpot;
	    }
	    if (windfall[n]) {
		if (windfall[n] >= 3000) nreport(n, N_WIN_LOTTO, n, 1);
		wu(0,n,fmt("POGOLOTTO win pays you $%d!\n",windfall[n]));
		np->nat_money += windfall[n];
	    }
	}
#endif
	/*
	** get per-population happiness and education
	** see what the total per-civilian production is
	** for this time period.
	*/
	np->nat_money += (int) (np->nat_reserve * money_res);
	hap = (levels[n][NAT_HLEV] * hap_edu) / ((float)pop / hap_cons);
	edu = levels[n][NAT_ELEV] / ((float)pop / edu_cons);
	wu((natid)0, n,
		fmt("%3.0f happiness, %3.0f education produced\n",
		levels[n][NAT_HLEV], levels[n][NAT_ELEV]));
	/* New tactic for preventing infinite education from
	 * low population.
	 */
	if (hap > (newhap = limit_edu(hap, etu))) {
		wu((natid)0, n,
		    fmt("%3.0f excess happiness produced.\n",
			hap - newhap));
		hap = newhap;
	}
	if (edu > (newedu = limit_edu(edu, etu))) {
		wu((natid)0, n,
		    fmt("%3.0f excess education produced.\n",
			edu - newedu));
		edu = newedu;
	}
	/*
	 * change the "moving average"...old happiness and
	 * education levels are weighted heavier than current
	 * production.
	 */
	newvalue = (np->nat_level[NAT_HLEV] * hap_avg + hap * etu) / 
		(hap_avg + etu);
	NAT_SETARY(nat_level[0], n, NAT_HLEV, newvalue);
	newvalue = (np->nat_level[NAT_ELEV] * edu_avg + edu * etu) / 
		(edu_avg + etu);
	NAT_SETARY(nat_level[0], n, NAT_ELEV, newvalue);
	/*
	 * limit tech/research production
	 */
	levels[n][NAT_TLEV] = limit_tech((double)levels[n][NAT_TLEV]);
	levels[n][NAT_RLEV] = limit_tech((double)levels[n][NAT_RLEV]);
	wu((natid)0, n,
	   fmt("total pop is %d, yielding %4.2f hap, %4.2f edu\n",
	       pop - 1, hap, edu));
    }
    if (ally_factor > 0.0)
	share_incr(res, tech);
    else {
	bzero(res, sizeof(res));
	bzero(tech, sizeof(tech));
    }
    for (n=0; np = getnatp(n); n++) {
	if ((np->nat_stat & STAT_NORM) == 0)
	    continue;
	tlev = levels[n][NAT_TLEV];
	rlev = levels[n][NAT_RLEV];
	if (tech[n] != 0.0 || res[n] != 0.0) {
	    wu((natid)0, n,
	       fmt("%5.4f technology (%5.4f + %5.4f), ",
		   tlev + tech[n], tlev, tech[n]));
	    wu((natid)0, n,
	       fmt("%5.4f research (%5.4f + %5.4f) produced\n",
		   rlev + res[n], rlev, res[n]));
	} else
	    wu((natid)0, n,
	       fmt("%5.4f tech, %5.4f research produced\n", tlev, rlev));
	rlev += res[n];
	tlev += tech[n];
	if (rlev != 0.0) NAT_DELTARY(nat_level[0], n, NAT_RLEV, rlev);
	if (tlev != 0.0) NAT_DELTARY(nat_level[0], n, NAT_TLEV, tlev);
	*lottobuf = '\0';
	if (sea_money[n]!=0) 
	    strcat(lottobuf,fmt("Navy delta $%d",sea_money[n]));
#ifdef PLANES
	if (air_money[n]!=0) {
	    if (*lottobuf) strcat(lottobuf,", ");
	    strcat(lottobuf,fmt("Air force delta $%d",air_money[n]));
	}
#endif
#ifdef SPIES
	if (spy_money[n]!=0) {
	    if (*lottobuf) strcat(lottobuf,", ");
	    strcat(lottobuf,fmt("Spy delta $%d",spy_money[n]));
	}
#endif
	if (*lottobuf) wu((natid)0, n, fmt("%s\n",lottobuf));
	wu((natid)0, n, fmt("money delta was $%d for this update\n",
			    np->nat_money - money[n]));
	NAT_DELTA(nat_money, n, np->nat_money - money[n]);
    }
}

/*
 * find out everyones increment
 */
share_incr(res, tech)
	register double *res;
	register double *tech;
{
	register struct natstr *np;
	register struct natstr *other;
	register natid i;
	register natid j;
	register int nc;

	for (i=0; np = getnatp(i); i++) {
		if ((np->nat_stat & STAT_INUSE) == 0)
			continue;
		nc = 0;
		res[i] = tech[i] = 0.0;
		for (j=0; other = getnatp(j); j++) {
			if (j == i)
				continue;
			if (getrel(np, j) != ALLIED)
				continue;
			if (getrel(other, i) != ALLIED)
				continue;
			res[i] += levels[j][NAT_RLEV];
			tech[i] += levels[j][NAT_TLEV];
			nc++;
		}
		if (nc == 0)
			continue;
		nc++;
		res[i] /= nc * ally_factor;
		tech[i] /= nc * ally_factor;
		logerror("Country #%d gets %g res and %g tec from %d allies",
			i, res[i], tech[i], nc - 1);
	}
}
