/* ________________________________________________________________
**|                                                                |
**| convship - convert ship database to include new information    |
**|                                                                |
**| If the ship data structure is ever modified, this utility      |
**| should be modified to reflect the old and new structures.      |
**| You can then type                                              |
**|                                                                |
**|     convship <OldShipDatabase> <NewShipDatabase>               |
**|                                                                |
**| to convert an existing ship database to the new structure.     |
**|________________________________________________________________|
*/

#include <fcntl.h>
typedef short coord;
typedef u_char natid;
extern int errno;

/* ________________________________________________________________
**|                                                                |
**| Old ship structure                                             |
**|________________________________________________________________|
*/

#define MAXSHPV	12

struct	oldshpstr {
	coord	shp_x;		/* x location in abs coords */
	coord	shp_y;		/* y location in abs coords */
	natid    shp_own;	/* owner's country num */
	char    shp_type;	/* ship type */
	char    shp_effic;	/* 0% to 100% */
	char    shp_mobil;	/* mobility units */
	short	shp_sell;	/* pointer to trade file */
	short	shp_tech;	/* tech level ship was built at */
	short	shp_uid;	/* unit id (ship #) */
	char    shp_fleet;	/* group membership */
	char    shp_nplane;	/* number of planes on board (max 8) */
	coord	shp_destx[2];	/* location for ship destination */
	coord	shp_desty[2];
	u_char	shp_trdtype[2];	/* commodity to trade at destination */
	u_char	shp_autonav;	/* autonavigation flags */
	char	shp_spare;	/* extra space */
	char    shp_nv;		/* current number of variables */
	u_char	shp_vtype[MAXSHPV];
	u_short	shp_vamt[MAXSHPV];
};

/* ________________________________________________________________
**|                                                                |
**| New ship structure                                             |
**|________________________________________________________________|
*/

struct	newshpstr {
	coord	shp_x;		/* x location in abs coords */
	coord	shp_y;		/* y location in abs coords */
	natid    shp_own;	/* owner's country num */
	char    shp_type;	/* ship type */
	char    shp_effic;	/* 0% to 100% */
	char    shp_mobil;	/* mobility units */
	short	shp_sell;	/* pointer to trade file */
	short	shp_tech;	/* tech level ship was built at */
	short	shp_uid;	/* unit id (ship #) */
	char    shp_fleet;	/* group membership */
	char    shp_nplane;	/* number of planes on board (max 8) */
	coord	shp_destx[2];	/* location for ship destination */
	coord	shp_desty[2];
	u_char	shp_trdtype[2];	/* commodity to trade at destination */
	u_char	shp_autonav;	/* autonavigation flags */
	short	shp_aboard;	/* if balloon/tank: ferry ship id */
	char	shp_spare;	/* extra space */
	char    shp_nv;		/* current number of variables */
	u_char	shp_vtype[MAXSHPV];
	u_short	shp_vamt[MAXSHPV];
};


#define oldsize sizeof(struct oldshpstr)
#define newsize sizeof(struct newshpstr)

struct oldshpstr oldship;
struct newshpstr newship;

main(argc,argv)
int argc;
char *argv[];
{
    int result;
    int done;
    int i,infile,outfile,numships;
    char *s,*d;
    
    if (argc<3) {
	printf("usage: convship <olddatabase> <newdatabase>\n");
	exit(0);
    }
    infile=open(argv[1],O_RDONLY);
    if (infile < 0) {
	printf("can't open '%s' for input\n",argv[1]);
	exit(0);
    }
    outfile=open(argv[2],O_CREAT | O_RDWR,0600);
    if (outfile < 0) {
	printf("can't create '%s' for output\n",argv[2]);
	exit(0);
    }
    
    printf ("Old structure size = %d bytes\n", oldsize);
    printf ("New structure size = %d bytes\n", newsize);
    
    for (done=numships=0; !done;) {
	result = read (infile, &oldship, oldsize);
	if (result != oldsize) {
	    if (result) printf ("error: read %d bytes, expected %d\n",result,oldsize);
	    done++;
	} else {

	    /* ________________________________________________________________
	    **|                                                                |
	    **| Copy old structure elements into new structure                 |
	    **| Initialize new structure elements not present in old structure |
	    **|________________________________________________________________|
	    */

	    newship.shp_x = oldship.shp_x;
	    newship.shp_y = oldship.shp_y;
	    newship.shp_own = oldship.shp_own;
	    newship.shp_type = oldship.shp_type;
	    newship.shp_effic = oldship.shp_effic;
	    newship.shp_mobil = oldship.shp_mobil;
	    newship.shp_sell = oldship.shp_sell;
	    newship.shp_tech = oldship.shp_tech;
	    newship.shp_uid = oldship.shp_uid;
	    newship.shp_fleet = oldship.shp_fleet;
	    newship.shp_nplane = oldship.shp_nplane;
	    newship.shp_destx[0] = oldship.shp_destx[0];
	    newship.shp_destx[1] = oldship.shp_destx[1];
	    newship.shp_desty[0] = oldship.shp_desty[0];
	    newship.shp_desty[1] = oldship.shp_desty[1];
	    newship.shp_trdtype[0] = oldship.shp_trdtype[0];
	    newship.shp_trdtype[1] = oldship.shp_trdtype[1];
	    newship.shp_autonav = oldship.shp_autonav;
	    newship.shp_aboard = -1;
	    newship.shp_spare = oldship.shp_spare;
	    newship.shp_nv = oldship.shp_nv;
	    for (i=0; i<MAXSHPV; i++)
		newship.shp_vtype[i] = oldship.shp_vtype[i];
	    for (i=0; i<MAXSHPV; i++)
		newship.shp_vamt[i] = oldship.shp_vamt[i];
	    
	    /* ________________________________________________________________
	    **|________________________________________________________________|
	    */

	    result = write (outfile, &newship, newsize);
	    if (result < 0) {
		printf ("error %d writing ship #%d\n",errno,numships);
		done++;
	    } else if (result != newsize) {
		printf("error: wrote %d bytes, expected %d\n",result,newsize);
		done++;
	    } else
		numships++;
	}
    }
	
    printf("%d ships processed\n",numships);
    printf("bytes read = %d\n",numships*oldsize);
    printf("bytes written = %d\n",numships*newsize);
    close(infile);
    close(outfile);
}
